# pylint: disable=line-too-long,useless-suppression
import functools
import pytest

from devtools_testutils import AzureRecordedTestCase, EnvironmentVariableLoader, recorded_by_proxy
from azure.core.credentials import AzureKeyCredential
from azure.ai.language.conversations.authoring import ConversationAuthoringClient

ConversationsPreparer = functools.partial(
    EnvironmentVariableLoader,
    "authoring",
    authoring_endpoint="https://Sanitized.cognitiveservices.azure.com/",
    authoring_key="fake_key",
)


class TestConversations(AzureRecordedTestCase):
    def create_client(self, endpoint, key):
        return ConversationAuthoringClient(endpoint, AzureKeyCredential(key))


class TestConversationsGetModelEvaluationSummarySync(TestConversations):
    @ConversationsPreparer()
    @recorded_by_proxy
    def test_get_model_evaluation_summary(self, authoring_endpoint, authoring_key):
        authoring_client = self.create_client(authoring_endpoint, authoring_key)

        project_name = "EmailApp"
        trained_model_label = "Model1"

        # Get trained-model scoped client and fetch the evaluation summary
        project_client = authoring_client.get_project_client(project_name)
        eval_summary = project_client.trained_model.get_model_evaluation_summary(trained_model_label)

        # Basic assertion the call returned something shaped like a summary
        assert eval_summary is not None

        # ----- Entities evaluation (micro/macro) -----
        entities_summary = eval_summary.entities_evaluation
        if entities_summary is not None:
            print(
                f"Entities - Micro F1: {entities_summary.micro_f1}, "
                f"Micro Precision: {entities_summary.micro_precision}, "
                f"Micro Recall: {entities_summary.micro_recall}"
            )
            print(
                f"Entities - Macro F1: {entities_summary.macro_f1}, "
                f"Macro Precision: {entities_summary.macro_precision}, "
                f"Macro Recall: {entities_summary.macro_recall}"
            )

            # Per-entity details
            ent_map = entities_summary.entities or {}
            for name, summary in ent_map.items():
                print(f"Entity '{name}': F1 = {summary.f1}, Precision = {summary.precision}, Recall = {summary.recall}")
                print(
                    f"  True Positives: {summary.true_positive_count}, "
                    f"True Negatives: {summary.true_negative_count}"
                )
                print(
                    f"  False Positives: {summary.false_positive_count}, "
                    f"False Negatives: {summary.false_negative_count}"
                )

        # ----- Intents evaluation (micro/macro) -----
        intents_summary = eval_summary.intents_evaluation
        if intents_summary is not None:
            print(
                f"Intents - Micro F1: {intents_summary.micro_f1}, "
                f"Micro Precision: {intents_summary.micro_precision}, "
                f"Micro Recall: {intents_summary.micro_recall}"
            )
            print(
                f"Intents - Macro F1: {intents_summary.macro_f1}, "
                f"Macro Precision: {intents_summary.macro_precision}, "
                f"Macro Recall: {intents_summary.macro_recall}"
            )

            # Per-intent details
            intent_map = intents_summary.intents or {}
            for name, summary in intent_map.items():
                print(f"Intent '{name}': F1 = {summary.f1}, Precision = {summary.precision}, Recall = {summary.recall}")
                print(
                    f"  True Positives: {summary.true_positive_count}, "
                    f"True Negatives: {summary.true_negative_count}"
                )
                print(
                    f"  False Positives: {summary.false_positive_count}, "
                    f"False Negatives: {summary.false_negative_count}"
                )



# coding=utf-8
# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------

"""
FILE: sample_get_model_evaluation_results_async.py
DESCRIPTION:
    This sample demonstrates how to retrieve detailed evaluation results (per utterance)
    for a trained model in a Conversation Authoring project (async).
USAGE:
    python sample_get_model_evaluation_results_async.py

REQUIRED ENV VARS (for AAD / DefaultAzureCredential):
    AZURE_CONVERSATIONS_AUTHORING_ENDPOINT
    AZURE_CLIENT_ID
    AZURE_TENANT_ID
    AZURE_CLIENT_SECRET

NOTE:
    If you want to use AzureKeyCredential instead, set:
      - AZURE_CONVERSATIONS_AUTHORING_ENDPOINT
      - AZURE_CONVERSATIONS_AUTHORING_KEY

OPTIONAL ENV VARS:
    PROJECT_NAME       # defaults to "<project-name>"
    TRAINED_MODEL      # defaults to "<trained-model-label>"
"""

# [START conversation_authoring_get_model_evaluation_results_async]
import os
import asyncio
from azure.identity import DefaultAzureCredential
from azure.ai.language.conversations.authoring.aio import ConversationAuthoringClient


async def sample_get_model_evaluation_results_async():
    # settings
    endpoint = os.environ["AZURE_CONVERSATIONS_AUTHORING_ENDPOINT"]
    project_name = os.environ.get("PROJECT_NAME", "<project-name>")
    trained_model_label = os.environ.get("TRAINED_MODEL", "<trained-model-label>")

    # async client with AAD
    credential = DefaultAzureCredential()
    async with ConversationAuthoringClient(endpoint, credential=credential) as client:
        project_client = client.get_project_client(project_name)

        # fetch async paged evaluation results
        results = project_client.trained_model.get_model_evaluation_results(
            trained_model_label=trained_model_label,
            string_index_type="Utf16CodeUnit",
        )

        print("=== Model Evaluation Results ===")
        seen = 0
        async for r in results:
            seen += 1
            print(f"Text: {r.text}")
            print(f"Language: {r.language}")

            if r.intents_result:
                print(f"Expected Intent: {r.intents_result.expected_intent}")
                print(f"Predicted Intent: {r.intents_result.predicted_intent}")

            if r.entities_result:
                expected_entities = r.entities_result.expected_entities or []
                print("Expected Entities:")
                for ent in expected_entities:
                    print(f" - Category: {ent.category}, Offset: {ent.offset}, Length: {ent.length}")

                predicted_entities = r.entities_result.predicted_entities or []
                print("Predicted Entities:")
                for ent in predicted_entities:
                    print(f" - Category: {ent.category}, Offset: {ent.offset}, Length: {ent.length}")

            print()

        print(f"Total Results: {seen}")


# [END conversation_authoring_get_model_evaluation_results_async]


async def main():
    await sample_get_model_evaluation_results_async()


if __name__ == "__main__":
    loop = asyncio.get_event_loop()
    loop.run_until_complete(main())



import pytest
from devtools_testutils import EnvironmentVariableLoader

# Environment variable keys
ENV_ENDPOINT = "AUTHORING_ENDPOINT"
ENV_KEY = "AUTHORING_KEY"

@pytest.fixture(scope="session")
def authoring_endpoint(environment_variables: EnvironmentVariableLoader) -> str:
    """Endpoint for Authoring tests."""
    return environment_variables.get(ENV_ENDPOINT)

@pytest.fixture(scope="session")
def authoring_key(environment_variables: EnvironmentVariableLoader) -> str:
    """API key for Authoring tests."""
    return environment_variables.get(ENV_KEY)

# autouse=True will trigger this fixture on each pytest run
# test_proxy auto-starts the test proxy
# patch_sleep and patch_async_sleep remove wait times during polling
@pytest.fixture(scope="session", autouse=True)
def start_proxy(test_proxy, patch_sleep, patch_async_sleep):
    return


# pylint: disable=line-too-long,useless-suppression
# coding=utf-8
# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------

"""
FILE: sample_get_model_evaluation_summary_async.py
DESCRIPTION:
    This sample demonstrates how to retrieve the evaluation summary for a trained model
    in a Conversation Authoring project (async).
USAGE:
    python sample_get_model_evaluation_summary_async.py

REQUIRED ENV VARS (for AAD / DefaultAzureCredential):
    AZURE_CONVERSATIONS_AUTHORING_ENDPOINT
    AZURE_CLIENT_ID
    AZURE_TENANT_ID
    AZURE_CLIENT_SECRET

NOTE:
    If you want to use AzureKeyCredential instead, set:
      - AZURE_CONVERSATIONS_AUTHORING_ENDPOINT
      - AZURE_CONVERSATIONS_AUTHORING_KEY

OPTIONAL ENV VARS:
    PROJECT_NAME   # defaults to "<project-name>"
    TRAINED_MODEL  # defaults to "<trained-model-label>"
"""

# [START conversation_authoring_get_model_evaluation_summary_async]
import os
import asyncio
from azure.identity import DefaultAzureCredential
from azure.ai.language.conversations.authoring.aio import ConversationAuthoringClient


async def sample_get_model_evaluation_summary_async():
    # settings
    endpoint = os.environ["AZURE_CONVERSATIONS_AUTHORING_ENDPOINT"]
    project_name = os.environ.get("PROJECT_NAME", "<project-name>")
    trained_model_label = os.environ.get("TRAINED_MODEL", "<trained-model-label>")

    credential = DefaultAzureCredential()
    async with ConversationAuthoringClient(endpoint, credential=credential) as client:
        project_client = client.get_project_client(project_name)

        # get evaluation summary for a trained model
        eval_summary = await project_client.trained_model.get_model_evaluation_summary(trained_model_label)

        print("=== Model Evaluation Summary ===")

        # ----- Entities evaluation (micro/macro) -----
        entities_summary = eval_summary.entities_evaluation
        if entities_summary is not None:
            print(
                f"Entities - Micro F1: {entities_summary.micro_f1}, "
                f"Micro Precision: {entities_summary.micro_precision}, "
                f"Micro Recall: {entities_summary.micro_recall}"
            )
            print(
                f"Entities - Macro F1: {entities_summary.macro_f1}, "
                f"Macro Precision: {entities_summary.macro_precision}, "
                f"Macro Recall: {entities_summary.macro_recall}"
            )

            # Per-entity details
            ent_map = entities_summary.entities or {}
            for entity_name, entity_summary in ent_map.items():
                print(
                    f"Entity '{entity_name}': F1 = {entity_summary.f1}, Precision = {entity_summary.precision}, Recall = {entity_summary.recall}"
                )
                print(
                    f"  True Positives: {entity_summary.true_positive_count}, "
                    f"True Negatives: {entity_summary.true_negative_count}"
                )
                print(
                    f"  False Positives: {entity_summary.false_positive_count}, "
                    f"False Negatives: {entity_summary.false_negative_count}"
                )

        # ----- Intents evaluation (micro/macro) -----
        intents_summary = eval_summary.intents_evaluation
        if intents_summary is not None:
            print(
                f"Intents - Micro F1: {intents_summary.micro_f1}, "
                f"Micro Precision: {intents_summary.micro_precision}, "
                f"Micro Recall: {intents_summary.micro_recall}"
            )
            print(
                f"Intents - Macro F1: {intents_summary.macro_f1}, "
                f"Macro Precision: {intents_summary.macro_precision}, "
                f"Macro Recall: {intents_summary.macro_recall}"
            )

            # Per-intent details
            intent_map = intents_summary.intents or {}
            for intent_name, intent_summary in intent_map.items():
                print(
                    f"Intent '{intent_name}': F1 = {intent_summary.f1}, Precision = {intent_summary.precision}, Recall = {intent_summary.recall}"
                )
                print(
                    f"  True Positives: {intent_summary.true_positive_count}, "
                    f"True Negatives: {intent_summary.true_negative_count}"
                )
                print(
                    f"  False Positives: {intent_summary.false_positive_count}, "
                    f"False Negatives: {intent_summary.false_negative_count}"
                )


# [END conversation_authoring_get_model_evaluation_summary_async]


async def main():
    await sample_get_model_evaluation_summary_async()


if __name__ == "__main__":
    loop = asyncio.get_event_loop()
    loop.run_until_complete(main())



# pylint: disable=line-too-long,useless-suppression
import functools
import pytest

from devtools_testutils import AzureRecordedTestCase, EnvironmentVariableLoader
from devtools_testutils.aio import recorded_by_proxy_async
from azure.core.credentials import AzureKeyCredential
from azure.ai.language.conversations.authoring.aio import ConversationAuthoringClient

ConversationsPreparer = functools.partial(
    EnvironmentVariableLoader,
    "authoring",
    authoring_endpoint="https://Sanitized.cognitiveservices.azure.com/",
    authoring_key="fake_key",
)


class TestConversationsAsync(AzureRecordedTestCase):
    async def create_client(self, endpoint: str, key: str) -> ConversationAuthoringClient:
        return ConversationAuthoringClient(endpoint, AzureKeyCredential(key))


class TestConversationsDeleteTrainedModelAsync(TestConversationsAsync):
    @ConversationsPreparer()
    @recorded_by_proxy_async
    @pytest.mark.asyncio
    async def test_delete_trained_model_async(self, authoring_endpoint, authoring_key):
        client = await self.create_client(authoring_endpoint, authoring_key)
        try:
            project_name = "Test-data-labels"
            trained_model_label = "MyModel"

            project_client = client.get_project_client(project_name)

            captured = {}

            def capture_response(pipeline_response):
                captured["status_code"] = pipeline_response.http_response.status_code

            # Act
            await project_client.trained_model.delete_trained_model(
                trained_model_label,
                raw_response_hook=capture_response,
            )

            # Print & Assert
            status = captured.get("status_code")
            print(f"Delete Trained Model Response Status: {status}")
            assert status == 204, f"Expected 204, got {status}"
        finally:
            await client.close()



# coding=utf-8
# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------

"""
FILE: sample_delete_trained_model_async.py
DESCRIPTION:
    This sample demonstrates how to delete a trained model from a Conversation Authoring project (async).
USAGE:
    python sample_delete_trained_model_async.py

REQUIRED ENV VARS (for AAD / DefaultAzureCredential):
    AZURE_CONVERSATIONS_AUTHORING_ENDPOINT
    AZURE_CLIENT_ID
    AZURE_TENANT_ID
    AZURE_CLIENT_SECRET

NOTE:
    If you want to use AzureKeyCredential instead, set:
      - AZURE_CONVERSATIONS_AUTHORING_ENDPOINT
      - AZURE_CONVERSATIONS_AUTHORING_KEY

OPTIONAL ENV VARS:
    PROJECT_NAME       # defaults to "<project-name>"
    TRAINED_MODEL      # defaults to "<trained-model-label>"
"""

# [START conversation_authoring_delete_trained_model_async]
import os
import asyncio
from azure.identity import DefaultAzureCredential
from azure.ai.language.conversations.authoring.aio import ConversationAuthoringClient


async def sample_delete_trained_model_async():
    # settings
    endpoint = os.environ["AZURE_CONVERSATIONS_AUTHORING_ENDPOINT"]
    project_name = os.environ.get("PROJECT_NAME", "<project-name>")
    trained_model_label = os.environ.get("TRAINED_MODEL", "<trained-model-label>")

    credential = DefaultAzureCredential()
    async with ConversationAuthoringClient(endpoint, credential=credential) as client:
        project_client = client.get_project_client(project_name)

        captured = {}

        def capture_response(pipeline_response):
            # capture the raw HTTP response status code
            captured["status_code"] = pipeline_response.http_response.status_code

        # delete trained model
        await project_client.trained_model.delete_trained_model(
            trained_model_label,
            raw_response_hook=capture_response,
        )

        # print response
        status = captured.get("status_code")
        print(f"Delete Trained Model Response Status: {status}")


# [END conversation_authoring_delete_trained_model_async]


async def main():
    await sample_delete_trained_model_async()


if __name__ == "__main__":
    loop = asyncio.get_event_loop()
    loop.run_until_complete(main())



# pylint: disable=line-too-long,useless-suppression
import functools

from devtools_testutils import AzureRecordedTestCase, EnvironmentVariableLoader, recorded_by_proxy
from azure.core.credentials import AzureKeyCredential
from azure.core.exceptions import HttpResponseError
from azure.ai.language.conversations.authoring import ConversationAuthoringClient
from azure.ai.language.conversations.authoring.models import ProjectResourceIds

ConversationsPreparer = functools.partial(
    EnvironmentVariableLoader,
    "authoring",
    authoring_endpoint="https://Sanitized.cognitiveservices.azure.com/",
    authoring_key="fake_key",
)


class TestConversations(AzureRecordedTestCase):
    def create_client(self, endpoint, key):
        return ConversationAuthoringClient(endpoint, AzureKeyCredential(key))


class TestConversationsDeleteDeploymentSync(TestConversations):
    @ConversationsPreparer()
    @recorded_by_proxy
    def test_delete_deployment_from_resources(self, authoring_endpoint, authoring_key):
        # Arrange
        client = self.create_client(authoring_endpoint, authoring_key)
        project_name = "EmailApp"
        deployment_name = "deploysdk2"
        project_client = client.get_project_client(project_name)

        # Create request body with the resource IDs to unassign/delete from
        delete_body = ProjectResourceIds(
            azure_resource_ids=[
                "/subscriptions/b72743ec-8bb3-453f-83ad-a53e8a50712e/resourceGroups/language-sdk-rg/providers/Microsoft.CognitiveServices/accounts/sdk-test-02"
            ]
        )

        # Act: begin delete and wait for completion
        poller = project_client.deployment.begin_delete_deployment_from_resources(
            deployment_name=deployment_name, body=delete_body
        )

        try:
            poller.result()
        except HttpResponseError as e:
            print(f"Operation failed: {e.message}")
            print(e.error)
            raise

        # Assert completion
        print(f"Delete completed. done={poller.done()} status={poller.status()}")



# coding=utf-8
# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------

"""
FILE: sample_get_model_evaluation_results.py
DESCRIPTION:
    This sample demonstrates how to retrieve detailed evaluation results (per utterance)
    for a trained model in a Conversation Authoring project.
USAGE:
    python sample_get_model_evaluation_results.py

REQUIRED ENV VARS (for AAD / DefaultAzureCredential):
    AZURE_CONVERSATIONS_AUTHORING_ENDPOINT
    AZURE_CLIENT_ID
    AZURE_TENANT_ID
    AZURE_CLIENT_SECRET

NOTE:
    If you want to use AzureKeyCredential instead, set:
      - AZURE_CONVERSATIONS_AUTHORING_ENDPOINT
      - AZURE_CONVERSATIONS_AUTHORING_KEY

OPTIONAL ENV VARS:
    PROJECT_NAME       # defaults to "<project-name>"
    TRAINED_MODEL      # defaults to "<trained-model-label>"
"""

# [START conversation_authoring_get_model_evaluation_results]
import os
from azure.identity import DefaultAzureCredential
from azure.ai.language.conversations.authoring import ConversationAuthoringClient


def sample_get_model_evaluation_results():
    # settings
    endpoint = os.environ["AZURE_CONVERSATIONS_AUTHORING_ENDPOINT"]
    project_name = os.environ.get("PROJECT_NAME", "<project-name>")
    trained_model_label = os.environ.get("TRAINED_MODEL", "<trained-model-label>")

    # create a client with AAD
    credential = DefaultAzureCredential()
    client = ConversationAuthoringClient(endpoint, credential=credential)
    project_client = client.get_project_client(project_name)

    # fetch paged evaluation results
    results = project_client.trained_model.get_model_evaluation_results(
        trained_model_label=trained_model_label, string_index_type="Utf16CodeUnit"
    )

    print("=== Model Evaluation Results ===")
    seen = 0
    for r in results:
        seen += 1
        print(f"Text: {r.text}")
        print(f"Language: {r.language}")

        if r.intents_result:
            print(f"Expected Intent: {r.intents_result.expected_intent}")
            print(f"Predicted Intent: {r.intents_result.predicted_intent}")

        if r.entities_result:
            expected_entities = r.entities_result.expected_entities or []
            print("Expected Entities:")
            for ent in expected_entities:
                print(f" - Category: {ent.category}, Offset: {ent.offset}, Length: {ent.length}")

            predicted_entities = r.entities_result.predicted_entities or []
            print("Predicted Entities:")
            for ent in predicted_entities:
                print(f" - Category: {ent.category}, Offset: {ent.offset}, Length: {ent.length}")

        print()

    print(f"Total Results: {seen}")


# [END conversation_authoring_get_model_evaluation_results]


def main():
    sample_get_model_evaluation_results()


if __name__ == "__main__":
    main()



# coding=utf-8
# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------

"""
FILE: sample_delete_project.py
DESCRIPTION:
    This sample demonstrates how to delete a Conversation Authoring project.
USAGE:
    python sample_delete_project.py

REQUIRED ENV VARS (for AAD / DefaultAzureCredential):
    AZURE_CONVERSATIONS_AUTHORING_ENDPOINT
    AZURE_CLIENT_ID
    AZURE_TENANT_ID
    AZURE_CLIENT_SECRET

NOTE:
    If you want to use AzureKeyCredential instead, set:
      - AZURE_CONVERSATIONS_AUTHORING_ENDPOINT
      - AZURE_CONVERSATIONS_AUTHORING_KEY

OPTIONAL ENV VARS:
    PROJECT_NAME   # defaults to "<project-name>"
"""

# [START conversation_authoring_delete_project]
import os
from azure.identity import DefaultAzureCredential
from azure.core.exceptions import HttpResponseError
from azure.ai.language.conversations.authoring import ConversationAuthoringClient


def sample_delete_project():
    # settings
    endpoint = os.environ["AZURE_CONVERSATIONS_AUTHORING_ENDPOINT"]
    project_name = os.environ.get("PROJECT_NAME", "<project-name>")

    # create a client with AAD
    credential = DefaultAzureCredential()
    client = ConversationAuthoringClient(endpoint, credential=credential)

    # start delete (long-running operation)
    poller = client.begin_delete_project(project_name=project_name)

    try:
        poller.result()
        print("Delete completed.")
        print(f"done: {poller.done()}")
        print(f"status: {poller.status()}")
    except HttpResponseError as e:
        print(f"Operation failed: {e.message}")
        print(e.error)


# [END conversation_authoring_delete_project]


def main():
    sample_delete_project()


if __name__ == "__main__":
    main()



# pylint: disable=line-too-long,useless-suppression
import functools
import pytest

from devtools_testutils import AzureRecordedTestCase, EnvironmentVariableLoader
from devtools_testutils.aio import recorded_by_proxy_async
from azure.core.credentials import AzureKeyCredential
from azure.ai.language.conversations.authoring.aio import ConversationAuthoringClient
from azure.ai.language.conversations.authoring.models import TrainingJobResult

ConversationsPreparer = functools.partial(
    EnvironmentVariableLoader,
    "authoring",
    authoring_endpoint="https://Sanitized.cognitiveservices.azure.com/",
    authoring_key="fake_key",
)


class TestConversationsAsync(AzureRecordedTestCase):
    async def create_client(self, endpoint: str, key: str) -> ConversationAuthoringClient:
        return ConversationAuthoringClient(endpoint, AzureKeyCredential(key))


class TestConversationsCancelTrainingAsync(TestConversationsAsync):
    @ConversationsPreparer()
    @recorded_by_proxy_async
    @pytest.mark.asyncio
    async def test_cancel_training_job_async(self, authoring_endpoint, authoring_key):
        client = await self.create_client(authoring_endpoint, authoring_key)
        try:
            project_name = "Test-data-labels"
            job_id = "122b7341-afaa-4bea-af63-f6c7e0539fcb_638985888000000000"
            project_client = client.get_project_client(project_name)

            poller = await project_client.project.begin_cancel_training_job(
                job_id=job_id,
            )

            result = await poller.result()  # TrainingJobResult

            assert (
                result.training_status.status == "cancelled"
            ), f"Cancellation failed with status: {result.training_status.status}"

            print(f"Model Label: {result.model_label}")
            print(f"Training Config Version: {result.training_config_version}")
            print(f"Training Mode: {result.training_mode}")
            if result.data_generation_status is not None:
                print(f"Data Generation Status: {result.data_generation_status.status}")
                print(f"Data Generation %: {result.data_generation_status.percent_complete}")
            if result.training_status is not None:
                print(f"Training Status: {result.training_status.status}")
                print(f"Training %: {result.training_status.percent_complete}")
            if result.evaluation_status is not None:
                print(f"Evaluation Status: {result.evaluation_status.status}")
                print(f"Evaluation %: {result.evaluation_status.percent_complete}")
            print(f"Estimated End: {result.estimated_end_on}")
        finally:
            await client.close()



# coding=utf-8
# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------

"""
FILE: sample_train_async.py
DESCRIPTION:
    This sample demonstrates how to start a training job for a Conversation Authoring project (async)
    and print the final training result.
USAGE:
    python sample_train_async.py

REQUIRED ENV VARS (for AAD / DefaultAzureCredential):
    AZURE_CONVERSATIONS_AUTHORING_ENDPOINT
    AZURE_CLIENT_ID
    AZURE_TENANT_ID
    AZURE_CLIENT_SECRET

NOTE:
    If you want to use AzureKeyCredential instead, set:
      - AZURE_CONVERSATIONS_AUTHORING_ENDPOINT
      - AZURE_CONVERSATIONS_AUTHORING_KEY

OPTIONAL ENV VARS:
    PROJECT_NAME  # defaults to "<project-name>"
"""

# [START conversation_authoring_train_async]
import os
import asyncio
from azure.identity import DefaultAzureCredential
from azure.ai.language.conversations.authoring.aio import ConversationAuthoringClient
from azure.ai.language.conversations.authoring.models import (
    TrainingJobDetails,
    TrainingMode,
    EvaluationDetails,
    EvaluationKind,
)


async def sample_train_async():
    # settings
    endpoint = os.environ["AZURE_CONVERSATIONS_AUTHORING_ENDPOINT"]
    project_name = os.environ.get("PROJECT_NAME", "<project-name>")

    # async client with AAD
    credential = DefaultAzureCredential()
    async with ConversationAuthoringClient(endpoint, credential=credential) as client:
        project_client = client.get_project_client(project_name)

        # build training request
        details = TrainingJobDetails(
            model_label="<model-label>",
            training_mode=TrainingMode.STANDARD,
            training_config_version="<config-version>",
            evaluation_options=EvaluationDetails(
                kind=EvaluationKind.PERCENTAGE,
                testing_split_percentage=20,
                training_split_percentage=80,
            ),
        )

        # start training job (async long-running operation)
        poller = await project_client.project.begin_train(body=details)

        # wait for job completion and get the result (no explicit type variables)
        result = await poller.result()

        # print result details (direct attribute access; no getattr)
        print("=== Training Result ===")
        print(f"Model Label: {result.model_label}")
        print(f"Training Config Version: {result.training_config_version}")
        print(f"Training Mode: {result.training_mode}")
        print(f"Training Status: {result.training_status}")
        print(f"Data Generation Status: {result.data_generation_status}")
        print(f"Evaluation Status: {result.evaluation_status}")
        print(f"Estimated End: {result.estimated_end_on}")


# [END conversation_authoring_train_async]


async def main():
    await sample_train_async()


if __name__ == "__main__":
    loop = asyncio.get_event_loop()
    loop.run_until_complete(main())



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from enum import Enum
from azure.core import CaseInsensitiveEnumMeta


class CompositionMode(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of CompositionMode."""

    RETURN_LONGEST_OVERLAP = "returnLongestOverlap"
    """When two or more components are found in the text and overlap, the component with the longest
    set of characters is returned."""
    REQUIRE_EXACT_OVERLAP = "requireExactOverlap"
    """All components must overlap at the exact same characters in the text for the entity to return.
    If one of the defined components is not matched or predicted, the entity will not return."""
    SEPARATE_COMPONENTS = "separateComponents"
    """Every component's match or prediction is returned as a separate instance of the entity."""
    COMBINE_COMPONENTS = "combineComponents"
    """When two or more components are found in the text and overlap, the components' spans are merged
    together into one span combining all of them."""


class DataGenerationConnectionKind(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Represents the connection kind for Azure OpenAI deployment."""

    AZURE_OPEN_AI = "AzureOpenAI"


class DatasetType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The dataset for this utterance. Allowed values are 'Train' and 'Test'."""

    TRAIN = "Train"
    """Split the data into training and test sets according to user-defined percentages."""
    TEST = "Test"
    """Split the data according to the chosen dataset for every example in the data."""


class EvaluationKind(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of EvaluationKind."""

    PERCENTAGE = "percentage"
    """Split the data into training and test sets according to user-defined percentages."""
    MANUAL = "manual"
    """Split the data according to the chosen dataset for every example in the data."""


class ExportedProjectFormat(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of ExportedProjectFormat."""

    CONVERSATION = "Conversation"
    """Specifies the format for a conversational project."""
    LUIS = "Luis"
    """Specifies the format for an application that was exported from LUIS."""


class OperationStatus(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of OperationStatus."""

    NOT_STARTED = "notStarted"
    RUNNING = "running"
    SUCCEEDED = "succeeded"
    FAILED = "failed"
    CANCELLED = "cancelled"
    CANCELLING = "cancelling"
    PARTIALLY_COMPLETED = "partiallyCompleted"


class OrchestrationTargetProjectKind(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of OrchestrationTargetProjectKind."""

    LUIS = "Luis"
    CONVERSATION = "Conversation"
    QUESTION_ANSWERING = "QuestionAnswering"


class ProjectKind(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of ProjectKind."""

    CONVERSATION = "Conversation"
    """A project to build natural language into apps, bots, and IoT devices."""
    ORCHESTRATION = "Orchestration"
    """A project to connect and orchestrate Conversation, Custom question answering and LUIS projects
    together in one single project."""
    CUSTOM_CONVERSATION_SUMMARIZATION = "CustomConversationSummarization"
    """A project to build conversation summarization models which are able to summarize long
    conversations."""


class StringIndexType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of StringIndexType."""

    UTF16_CODE_UNIT = "Utf16CodeUnit"
    """The offset and length values will correspond to UTF-16 code units. Use this option if your
    application is written in a language that support Unicode, for example Java, JavaScript."""
    UTF8_CODE_UNIT = "Utf8CodeUnit"
    UTF32_CODE_UNIT = "Utf32CodeUnit"


class TrainingMode(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of TrainingMode."""

    ADVANCED = "advanced"
    """Trains using fine-tuned neural network transformer models. Can train multilingual projects."""
    STANDARD = "standard"
    """Faster training times for quicker iterations."""



# pylint: disable=line-too-long,useless-suppression
import functools
from devtools_testutils import AzureRecordedTestCase, EnvironmentVariableLoader, recorded_by_proxy
from azure.core.credentials import AzureKeyCredential
from azure.ai.language.conversations.authoring import ConversationAuthoringClient

ConversationsPreparer = functools.partial(
    EnvironmentVariableLoader,
    "authoring",
    authoring_endpoint="https://Sanitized.cognitiveservices.azure.com/",
    authoring_key="fake_key",
)


class TestConversations(AzureRecordedTestCase):
    def create_client(self, endpoint, key):
        return ConversationAuthoringClient(endpoint, AzureKeyCredential(key))


class TestConversationsDeleteTrainedModelSync(TestConversations):
    @ConversationsPreparer()
    @recorded_by_proxy
    def test_delete_trained_model(self, authoring_endpoint, authoring_key):
        client = self.create_client(authoring_endpoint, authoring_key)

        project_name = "Test-data-labels"
        trained_model_label = "model1"

        project_client = client.get_project_client(project_name)

        captured = {}

        def capture_response(pipeline_response):
            captured["status_code"] = pipeline_response.http_response.status_code

        # Act
        project_client.trained_model.delete_trained_model(trained_model_label, raw_response_hook=capture_response)

        # Print & Assert
        status = captured.get("status_code")
        print(f"Delete Trained Model Response Status: {status}")
        assert status == 204, f"Expected 204, got {status}"



# pylint: disable=line-too-long,useless-suppression
import functools
import pytest
from devtools_testutils import AzureRecordedTestCase, EnvironmentVariableLoader
from devtools_testutils.aio import recorded_by_proxy_async
from azure.core.credentials import AzureKeyCredential
from azure.ai.language.conversations.authoring.aio import ConversationAuthoringClient

ConversationsPreparer = functools.partial(
    EnvironmentVariableLoader,
    "authoring",
    authoring_endpoint="https://Sanitized.cognitiveservices.azure.com/",
    authoring_key="fake_key",
)


class TestConversationsAsync(AzureRecordedTestCase):
    async def create_client(self, endpoint: str, key: str) -> ConversationAuthoringClient:
        return ConversationAuthoringClient(endpoint, AzureKeyCredential(key))


class TestConversationsGetDeploymentAsync(TestConversationsAsync):
    @ConversationsPreparer()
    @recorded_by_proxy_async
    @pytest.mark.asyncio
    async def test_get_deployment_async(self, authoring_endpoint, authoring_key):
        client = await self.create_client(authoring_endpoint, authoring_key)
        try:
            project_name = "EmailApp"
            deployment_name = "staging"

            project_client = client.get_project_client(project_name)

            # Act
            deployment = await project_client.deployment.get_deployment(deployment_name=deployment_name)

            # Basic assertion
            assert deployment is not None

            # Print like the C# sample (snake_case properties in Python)
            print(f"Deployment Name: {getattr(deployment, 'deployment_name', None)}")
            print(f"Model Id: {getattr(deployment, 'model_id', None)}")
            print(f"Last Trained On: {getattr(deployment, 'last_trained_on', None)}")
            print(f"Last Deployed On: {getattr(deployment, 'last_deployed_on', None)}")
            print(f"Deployment Expired On: {getattr(deployment, 'deployment_expired_on', None)}")
            print(f"Model Training Config Version: {getattr(deployment, 'model_training_config_version', None)}")

            assigned_resources = getattr(deployment, "assigned_resources", None)
            if assigned_resources:
                for ar in assigned_resources:
                    print(f"Resource ID: {getattr(ar, 'resource_id', None)}")
                    print(f"Region: {getattr(ar, 'region', None)}")
                    aoai = getattr(ar, "assigned_aoai_resource", None)
                    if aoai:
                        print(f"AOAI Kind: {getattr(aoai, 'kind', None)}")
                        print(f"AOAI Resource ID: {getattr(aoai, 'resource_id', None)}")
                        print(f"AOAI Deployment Name: {getattr(aoai, 'deployment_name', None)}")
        finally:
            await client.close()



# coding=utf-8
# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------

"""
FILE: sample_get_deployment_async.py
DESCRIPTION:
    This sample demonstrates how to retrieve details of a deployment in a Conversation Authoring project (async).
USAGE:
    python sample_get_deployment_async.py

REQUIRED ENV VARS (for AAD / DefaultAzureCredential):
    AZURE_CONVERSATIONS_AUTHORING_ENDPOINT
    AZURE_CLIENT_ID
    AZURE_TENANT_ID
    AZURE_CLIENT_SECRET

NOTE:
    If you want to use AzureKeyCredential instead, set:
      - AZURE_CONVERSATIONS_AUTHORING_ENDPOINT
      - AZURE_CONVERSATIONS_AUTHORING_KEY

OPTIONAL ENV VARS:
    PROJECT_NAME      # defaults to "<project-name>"
    DEPLOYMENT_NAME   # defaults to "<deployment-name>"
"""

# [START conversation_authoring_get_deployment_async]
import os
import asyncio
from azure.identity.aio import DefaultAzureCredential
from azure.ai.language.conversations.authoring.aio import ConversationAuthoringClient


async def sample_get_deployment_async():
    # settings
    endpoint = os.environ["AZURE_CONVERSATIONS_AUTHORING_ENDPOINT"]
    project_name = os.environ.get("PROJECT_NAME", "<project-name>")
    deployment_name = os.environ.get("DEPLOYMENT_NAME", "<deployment-name>")

    credential = DefaultAzureCredential()
    async with ConversationAuthoringClient(endpoint, credential=credential) as client:
        project_client = client.get_project_client(project_name)

        # get deployment details
        deployment = await project_client.deployment.get_deployment(deployment_name=deployment_name)

        print("=== Deployment Details ===")
        print(f"Deployment Name: {deployment.deployment_name}")
        print(f"Model Id: {deployment.model_id}")
        print(f"Last Trained On: {deployment.last_trained_on}")
        print(f"Last Deployed On: {deployment.last_deployed_on}")
        print(f"Deployment Expired On: {deployment.deployment_expired_on}")
        print(f"Model Training Config Version: {deployment.model_training_config_version}")

        # print assigned resources
        if deployment.assigned_resources:
            for ar in deployment.assigned_resources:
                print(f"Resource ID: {ar.resource_id}")
                print(f"Region: {ar.region}")
                if ar.assigned_aoai_resource:
                    aoai = ar.assigned_aoai_resource
                    print(f"AOAI Kind: {aoai.kind}")
                    print(f"AOAI Resource ID: {aoai.resource_id}")
                    print(f"AOAI Deployment Name: {aoai.deployment_name}")


# [END conversation_authoring_get_deployment_async]


async def main():
    await sample_get_deployment_async()


if __name__ == "__main__":
    loop = asyncio.get_event_loop()
    loop.run_until_complete(main())



# pylint: disable=line-too-long,useless-suppression
# coding=utf-8
# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------

"""
FILE: sample_get_assign_project_resources_status_async.py
DESCRIPTION:
    This sample demonstrates how to get the status of a project resource-assignment job in a Conversation Authoring project (async).

USAGE:
    python sample_get_assign_project_resources_status_async.py

REQUIRED ENV VARS (for AAD / DefaultAzureCredential):
    AZURE_CONVERSATIONS_AUTHORING_ENDPOINT
    AZURE_CLIENT_ID
    AZURE_TENANT_ID
    AZURE_CLIENT_SECRET

NOTE:
    If you want to use AzureKeyCredential instead, set:
      - AZURE_CONVERSATIONS_AUTHORING_ENDPOINT
      - AZURE_CONVERSATIONS_AUTHORING_KEY

OPTIONAL ENV VARS:
    PROJECT_NAME         # defaults to "<project-name>"
"""

# [START conversation_authoring_get_assign_project_resources_status_async]
import os
import asyncio
from azure.identity.aio import DefaultAzureCredential
from azure.core.exceptions import HttpResponseError
from azure.ai.language.conversations.authoring.aio import ConversationAuthoringClient
from azure.ai.language.conversations.authoring.models import ProjectResourcesState


async def sample_get_assign_project_resources_status_async():
    # settings
    endpoint = os.environ["AZURE_CONVERSATIONS_AUTHORING_ENDPOINT"]
    project_name = os.environ.get("PROJECT_NAME", "<project-name>")

    # Example job ID (replace with real one returned by assign operation)
    job_id = "00000000-0000-0000-0000-000000000000"

    credential = DefaultAzureCredential()
    async with ConversationAuthoringClient(endpoint, credential=credential) as client:
        project_client = client.get_project_client(project_name)

        # start get-status call
        try:
            state = await project_client.project.get_assign_project_resources_status(job_id)
            print("Status retrieved successfully.")
            print(f"Job ID: {state.job_id}")
            print(f"Status: {state.status}")
            print(f"Created On: {state.created_on}")
            print(f"Last Updated On: {state.last_updated_on}")
        except HttpResponseError as e:
            print(f"Operation failed: {e.message}")
            print(e.error)


# [END conversation_authoring_get_assign_project_resources_status_async]


async def main():
    await sample_get_assign_project_resources_status_async()


if __name__ == "__main__":
    loop = asyncio.get_event_loop()
    loop.run_until_complete(main())



# pylint: disable=line-too-long,useless-suppression
import functools
import json

from devtools_testutils import AzureRecordedTestCase, EnvironmentVariableLoader, recorded_by_proxy
from azure.core.credentials import AzureKeyCredential
from azure.core.exceptions import HttpResponseError
from azure.ai.language.conversations.authoring import ConversationAuthoringClient
from azure.ai.language.conversations.authoring.models import (
    CreateProjectOptions,
    ProjectKind,
    ProjectSettings,
    ExportedProject,
    ExportedProjectFormat,
    ConversationExportedProjectAsset,
    ConversationExportedIntent,
    ConversationExportedEntity,
    ConversationExportedUtterance,
    ExportedUtteranceEntityLabel,
)

ConversationsPreparer = functools.partial(
    EnvironmentVariableLoader,
    "authoring",
    authoring_endpoint="https://Sanitized.cognitiveservices.azure.com/",
    authoring_key="fake_key",
)


class TestConversations(AzureRecordedTestCase):
    # Start with any helper functions you might need, for example a client creation method:
    def create_client(self, endpoint, key):
        credential = AzureKeyCredential(key)
        client = ConversationAuthoringClient(endpoint, credential)
        return client

    # ...


class TestConversationsImportCase(TestConversations):
    @ConversationsPreparer()
    @recorded_by_proxy
    def test_import_project(self, authoring_endpoint, authoring_key):
        authoring_client = self.create_client(authoring_endpoint, authoring_key)
        project_name = "PythonImportProject1113"
        project_client = authoring_client.get_project_client(project_name)

        # ----- Build assets using objects -----
        intents = [
            ConversationExportedIntent(category="None"),
            ConversationExportedIntent(category="Buy"),
        ]

        entities = [
            ConversationExportedEntity(
                category="product",
                composition_mode="combineComponents",
            )
        ]

        u1 = ConversationExportedUtterance(
            text="I want to buy a house",
            intent="Buy",
            language="en-us",
            dataset="Train",
            entities=[ExportedUtteranceEntityLabel(category="product", offset=16, length=5)],
        )

        u2 = ConversationExportedUtterance(
            text="I want to buy surface pro",
            intent="Buy",
            language="en-us",
            dataset="Train",
            entities=[ExportedUtteranceEntityLabel(category="product", offset=14, length=11)],
        )

        u3 = ConversationExportedUtterance(
            text="I want to buy xbox",
            intent="Buy",
            language="en-us",
            dataset="Train",
            entities=[ExportedUtteranceEntityLabel(category="product", offset=14, length=4)],
        )

        assets = ConversationExportedProjectAsset(
            intents=intents,
            entities=entities,
            utterances=[u1, u2, u3],
        )

        metadata = CreateProjectOptions(
            project_kind=ProjectKind.CONVERSATION,
            project_name=project_name,  # required
            language="en-us",  # required
            settings=ProjectSettings(confidence_threshold=0.0),
            multilingual=False,
            description="",
        )

        exported_project = ExportedProject(
            project_file_version="2025-05-15-preview",
            string_index_type="Utf16CodeUnit",
            metadata=metadata,
            assets=assets,
        )

        # ----- Act: begin import (LRO) -----
        poller = project_client.project.begin_import(
            body=exported_project,
            exported_project_format=ExportedProjectFormat.CONVERSATION,
        )

        try:
            poller.result()
        except HttpResponseError as e:
            print(f"Operation failed: {e.message}")
            print(e.error)
            raise

        print(f"Import completed. done={poller.done()} status={poller.status()}")



# pylint: disable=line-too-long,useless-suppression
import functools

from devtools_testutils import AzureRecordedTestCase, EnvironmentVariableLoader, recorded_by_proxy
from azure.core.credentials import AzureKeyCredential
from azure.ai.language.conversations.authoring import ConversationAuthoringClient
from azure.ai.language.conversations.authoring.models import (
    TrainingJobDetails,
    TrainingMode,
    EvaluationDetails,
    EvaluationKind,
    TrainingJobResult,
)

ConversationsPreparer = functools.partial(
    EnvironmentVariableLoader,
    "authoring",
    authoring_endpoint="https://Sanitized.cognitiveservices.azure.com/",
    authoring_key="fake_key",
)


class TestConversations(AzureRecordedTestCase):
    def create_client(self, endpoint, key):
        return ConversationAuthoringClient(endpoint, AzureKeyCredential(key))


class TestConversationsTrainSync(TestConversations):
    @ConversationsPreparer()
    @recorded_by_proxy
    def test_train_project_sync(self, authoring_endpoint, authoring_key):
        client = self.create_client(authoring_endpoint, authoring_key)
        project_name = "Test-data-labels"
        project_client = client.get_project_client(project_name)

        # Arrange training request (object model; snake_case fields)
        training_job_details = TrainingJobDetails(
            model_label="MyModel",
            training_mode=TrainingMode.STANDARD,
            training_config_version="2023-04-15",
            evaluation_options=EvaluationDetails(
                kind=EvaluationKind.PERCENTAGE,
                testing_split_percentage=20,
                training_split_percentage=80,
            ),
        )

        # Act: begin train (LRO)
        poller = project_client.project.begin_train(body=training_job_details)

        # Wait for completion and get the TrainingJobResult
        result: TrainingJobResult = poller.result()

        # Prints for visibility
        print(f"Model Label: {result.model_label}")
        print(f"Training Config Version: {result.training_config_version}")
        print(f"Training Mode: {result.training_mode}")
        print(f"Training Status: {result.training_status}")
        print(f"Data Generation Status: {result.data_generation_status}")
        print(f"Evaluation Status: {result.evaluation_status}")
        print(f"Estimated End: {result.estimated_end_on}")



# coding=utf-8
# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------

"""
FILE: sample_create_project_async.py
DESCRIPTION:
    This sample demonstrates how to create a Conversation Authoring project (async).
USAGE:
    python sample_create_project_async.py

REQUIRED ENV VARS (for AAD / DefaultAzureCredential):
    AZURE_CONVERSATIONS_AUTHORING_ENDPOINT
    AZURE_CLIENT_ID
    AZURE_TENANT_ID
    AZURE_CLIENT_SECRET

NOTE:
    If you want to use AzureKeyCredential instead, set:
      - AZURE_CONVERSATIONS_AUTHORING_ENDPOINT
      - AZURE_CONVERSATIONS_AUTHORING_KEY

OPTIONAL ENV VARS:
    PROJECT_NAME   # defaults to "<project-name>"
"""

# [START conversation_authoring_create_project_async]
import os
import asyncio
from azure.identity.aio import DefaultAzureCredential
from azure.ai.language.conversations.authoring.aio import ConversationAuthoringClient
from azure.ai.language.conversations.authoring.models import (
    CreateProjectOptions,
    ProjectKind,
)


async def sample_create_project_async():
    # settings
    endpoint = os.environ["AZURE_CONVERSATIONS_AUTHORING_ENDPOINT"]
    project_name = os.environ.get("PROJECT_NAME", "<project-name>")

    credential = DefaultAzureCredential()
    async with ConversationAuthoringClient(endpoint, credential=credential) as client:
        # build project definition
        body = CreateProjectOptions(
            project_kind=ProjectKind.CONVERSATION,
            project_name=project_name,
            language="<language-tag>",  # e.g. "en-us"
            multilingual=True,
            description="Sample project created via Python SDK",
        )

        # create project (no explicit type variables)
        result = await client.create_project(project_name=project_name, body=body)

        # print project details (direct attribute access; no getattr)
        print("=== Create Project Result ===")
        print(f"Project Name: {result.project_name}")
        print(f"Language: {result.language}")
        print(f"Kind: {result.project_kind}")
        print(f"Multilingual: {result.multilingual}")
        print(f"Description: {result.description}")


# [END conversation_authoring_create_project_async]


async def main():
    await sample_create_project_async()


if __name__ == "__main__":
    loop = asyncio.get_event_loop()
    loop.run_until_complete(main())



# pylint: disable=line-too-long,useless-suppression
import functools
import pytest

from devtools_testutils import AzureRecordedTestCase, EnvironmentVariableLoader
from devtools_testutils.aio import recorded_by_proxy_async
from azure.core.credentials import AzureKeyCredential
from azure.ai.language.conversations.authoring.aio import ConversationAuthoringClient

ConversationsPreparer = functools.partial(
    EnvironmentVariableLoader,
    "authoring",
    authoring_endpoint="https://Sanitized.cognitiveservices.azure.com/",
    authoring_key="fake_key",
)


class TestConversationsAsync(AzureRecordedTestCase):
    async def create_client(self, endpoint: str, key: str) -> ConversationAuthoringClient:
        return ConversationAuthoringClient(endpoint, AzureKeyCredential(key))


class TestConversationsGetModelEvaluationSummaryAsync(TestConversationsAsync):
    @ConversationsPreparer()
    @recorded_by_proxy_async
    @pytest.mark.asyncio
    async def test_get_model_evaluation_summary_async(self, authoring_endpoint, authoring_key):
        client = await self.create_client(authoring_endpoint, authoring_key)
        try:
            project_name = "EmailApp"
            trained_model_label = "Model1"

            # Get trained-model scoped client and fetch the evaluation summary
            project_client = client.get_project_client(project_name)
            eval_summary = await project_client.trained_model.get_model_evaluation_summary(trained_model_label)

            # Basic assertion
            assert eval_summary is not None

            # ----- Entities evaluation (micro/macro) -----
            entities_eval = getattr(eval_summary, "entities_evaluation", None)
            if entities_eval:
                print(
                    f"Entities - Micro F1: {getattr(entities_eval, 'micro_f1', None)}, Micro Precision: {getattr(entities_eval, 'micro_precision', None)}, Micro Recall: {getattr(entities_eval, 'micro_recall', None)}"
                )
                print(
                    f"Entities - Macro F1: {getattr(entities_eval, 'macro_f1', None)}, Macro Precision: {getattr(entities_eval, 'macro_precision', None)}, Macro Recall: {getattr(entities_eval, 'macro_recall', None)}"
                )

                ent_map = getattr(entities_eval, "entities", {}) or {}
                for name, summary in ent_map.items():
                    print(
                        f"Entity '{name}': F1 = {getattr(summary, 'f1', None)}, Precision = {getattr(summary, 'precision', None)}, Recall = {getattr(summary, 'recall', None)}"
                    )
                    print(
                        f"  True Positives: {getattr(summary, 'true_positive_count', None)}, True Negatives: {getattr(summary, 'true_negative_count', None)}"
                    )
                    print(
                        f"  False Positives: {getattr(summary, 'false_positive_count', None)}, False Negatives: {getattr(summary, 'false_negative_count', None)}"
                    )

            # ----- Intents evaluation (micro/macro) -----
            intents_eval = getattr(eval_summary, "intents_evaluation", None)
            if intents_eval:
                print(
                    f"Intents - Micro F1: {getattr(intents_eval, 'micro_f1', None)}, Micro Precision: {getattr(intents_eval, 'micro_precision', None)}, Micro Recall: {getattr(intents_eval, 'micro_recall', None)}"
                )
                print(
                    f"Intents - Macro F1: {getattr(intents_eval, 'macro_f1', None)}, Macro Precision: {getattr(intents_eval, 'macro_precision', None)}, Macro Recall: {getattr(intents_eval, 'macro_recall', None)}"
                )

                intent_map = getattr(intents_eval, "intents", {}) or {}
                for name, summary in intent_map.items():
                    print(
                        f"Intent '{name}': F1 = {getattr(summary, 'f1', None)}, Precision = {getattr(summary, 'precision', None)}, Recall = {getattr(summary, 'recall', None)}"
                    )
                    print(
                        f"  True Positives: {getattr(summary, 'true_positive_count', None)}, True Negatives: {getattr(summary, 'true_negative_count', None)}"
                    )
                    print(
                        f"  False Positives: {getattr(summary, 'false_positive_count', None)}, False Negatives: {getattr(summary, 'false_negative_count', None)}"
                    )
        finally:
            await client.close()



# coding=utf-8
# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------

"""
FILE: sample_get_deployment.py
DESCRIPTION:
    This sample demonstrates how to retrieve details of a deployment in a Conversation Authoring project.
USAGE:
    python sample_get_deployment.py

REQUIRED ENV VARS (for AAD / DefaultAzureCredential):
    AZURE_CONVERSATIONS_AUTHORING_ENDPOINT
    AZURE_CLIENT_ID
    AZURE_TENANT_ID
    AZURE_CLIENT_SECRET

NOTE:
    If you want to use AzureKeyCredential instead, set:
      - AZURE_CONVERSATIONS_AUTHORING_ENDPOINT
      - AZURE_CONVERSATIONS_AUTHORING_KEY

OPTIONAL ENV VARS:
    PROJECT_NAME      # defaults to "<project-name>"
    DEPLOYMENT_NAME   # defaults to "<deployment-name>"
"""

# [START conversation_authoring_get_deployment]
import os
from azure.identity import DefaultAzureCredential
from azure.ai.language.conversations.authoring import ConversationAuthoringClient


def sample_get_deployment():
    # settings
    endpoint = os.environ["AZURE_CONVERSATIONS_AUTHORING_ENDPOINT"]
    project_name = os.environ.get("PROJECT_NAME", "<project-name>")
    deployment_name = os.environ.get("DEPLOYMENT_NAME", "<deployment-name>")

    # create a client with AAD
    credential = DefaultAzureCredential()
    client = ConversationAuthoringClient(endpoint, credential=credential)
    project_client = client.get_project_client(project_name)

    # get deployment details
    deployment = project_client.deployment.get_deployment(deployment_name=deployment_name)

    print("=== Deployment Details ===")
    print(f"Deployment Name: {deployment.deployment_name}")
    print(f"Model Id: {deployment.model_id}")
    print(f"Last Trained On: {deployment.last_trained_on}")
    print(f"Last Deployed On: {deployment.last_deployed_on}")
    print(f"Deployment Expired On: {deployment.deployment_expired_on}")
    print(f"Model Training Config Version: {deployment.model_training_config_version}")

    # print assigned resources (direct attribute access; no getattr)
    if deployment.assigned_resources:
        for ar in deployment.assigned_resources:
            print(f"Resource ID: {ar.resource_id}")
            print(f"Region: {ar.region}")
            if ar.assigned_aoai_resource:
                aoai = ar.assigned_aoai_resource
                print(f"AOAI Kind: {aoai.kind}")
                print(f"AOAI Resource ID: {aoai.resource_id}")
                print(f"AOAI Deployment Name: {aoai.deployment_name}")


# [END conversation_authoring_get_deployment]


def main():
    sample_get_deployment()


if __name__ == "__main__":
    main()



# coding=utf-8
# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------

"""
FILE: sample_cancel_training_job.py
DESCRIPTION:
    This sample demonstrates how to cancel a training job in a Conversation Authoring project.
USAGE:
    python sample_cancel_training_job.py

REQUIRED ENV VARS (for AAD / DefaultAzureCredential):
    AZURE_CONVERSATIONS_AUTHORING_ENDPOINT
    AZURE_CLIENT_ID
    AZURE_TENANT_ID
    AZURE_CLIENT_SECRET

NOTE:
    If you want to use AzureKeyCredential instead, set:
      - AZURE_CONVERSATIONS_AUTHORING_ENDPOINT
      - AZURE_CONVERSATIONS_AUTHORING_KEY

OPTIONAL ENV VARS:
    PROJECT_NAME   # defaults to "<project-name>"
    JOB_ID         # defaults to "<job-id>"
"""

# [START conversation_authoring_cancel_training_job]
import os
from azure.identity import DefaultAzureCredential
from azure.ai.language.conversations.authoring import ConversationAuthoringClient


def sample_cancel_training_job():
    # settings
    endpoint = os.environ["AZURE_CONVERSATIONS_AUTHORING_ENDPOINT"]
    project_name = os.environ.get("PROJECT_NAME", "<project-name>")
    job_id = os.environ.get("JOB_ID", "<job-id>")

    # create a client with AAD
    credential = DefaultAzureCredential()
    client = ConversationAuthoringClient(endpoint, credential=credential)
    project_client = client.get_project_client(project_name)

    poller = project_client.project.begin_cancel_training_job(
        job_id=job_id,
    )

    result = poller.result()  # TrainingJobResult
    print(f"Model Label: {result.model_label}")
    print(f"Training Config Version: {result.training_config_version}")
    print(f"Training Mode: {result.training_mode}")
    if result.data_generation_status is not None:
        print(f"Data Generation Status: {result.data_generation_status.status}")
        print(f"Data Generation %: {result.data_generation_status.percent_complete}")
    if result.training_status is not None:
        print(f"Training Status: {result.training_status.status}")
        print(f"Training %: {result.training_status.percent_complete}")
    if result.evaluation_status is not None:
        print(f"Evaluation Status: {result.evaluation_status.status}")
        print(f"Evaluation %: {result.evaluation_status.percent_complete}")
    print(f"Estimated End: {result.estimated_end_on}")


# [END conversation_authoring_cancel_training_job]


def main():
    sample_cancel_training_job()


if __name__ == "__main__":
    main()



# coding=utf-8
# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------

"""
FILE: sample_get_project_async.py
DESCRIPTION:
    This sample demonstrates how to retrieve details of a Conversation Authoring project (async).
USAGE:
    python sample_get_project_async.py

REQUIRED ENV VARS (for AAD / DefaultAzureCredential):
    AZURE_CONVERSATIONS_AUTHORING_ENDPOINT
    AZURE_CLIENT_ID
    AZURE_TENANT_ID
    AZURE_CLIENT_SECRET

NOTE:
    If you want to use AzureKeyCredential instead, set:
      - AZURE_CONVERSATIONS_AUTHORING_ENDPOINT
      - AZURE_CONVERSATIONS_AUTHORING_KEY

OPTIONAL ENV VARS:
    PROJECT_NAME   # defaults to "<project-name>"
"""

# [START conversation_authoring_get_project_async]
import os
import asyncio
from azure.identity.aio import DefaultAzureCredential
from azure.ai.language.conversations.authoring.aio import ConversationAuthoringClient


async def sample_get_project_async():
    # settings
    endpoint = os.environ["AZURE_CONVERSATIONS_AUTHORING_ENDPOINT"]
    project_name = os.environ.get("PROJECT_NAME", "<project-name>")

    credential = DefaultAzureCredential()
    async with ConversationAuthoringClient(endpoint, credential=credential) as client:
        # get project details (no explicit type variables)
        details = await client.get_project(project_name=project_name)

        # print result details (direct attribute access)
        print("=== Project Details ===")
        print(f"Created DateTime: {details.created_on}")
        print(f"Last Modified DateTime: {details.last_modified_on}")
        print(f"Last Trained DateTime: {details.last_trained_on}")
        print(f"Last Deployed DateTime: {details.last_deployed_on}")
        print(f"Project Kind: {details.project_kind}")
        print(f"Project Name: {details.project_name}")
        print(f"Multilingual: {details.multilingual}")
        print(f"Description: {details.description}")
        print(f"Language: {details.language}")


# [END conversation_authoring_get_project_async]


async def main():
    await sample_get_project_async()


if __name__ == "__main__":
    loop = asyncio.get_event_loop()
    loop.run_until_complete(main())



# pylint: disable=line-too-long,useless-suppression
import functools
from devtools_testutils import AzureRecordedTestCase, EnvironmentVariableLoader, recorded_by_proxy
from azure.core.credentials import AzureKeyCredential
from azure.core.exceptions import HttpResponseError
from azure.ai.language.conversations.authoring import ConversationAuthoringClient
from azure.ai.language.conversations.authoring.models import (
    SwapDeploymentsDetails,
)

ConversationsPreparer = functools.partial(
    EnvironmentVariableLoader,
    "authoring",
    authoring_endpoint="https://Sanitized.cognitiveservices.azure.com/",
    authoring_key="fake_key",
)


class TestConversations(AzureRecordedTestCase):
    def create_client(self, endpoint, key):
        return ConversationAuthoringClient(endpoint, AzureKeyCredential(key))


class TestConversationsSwapDeploymentsSync(TestConversations):
    @ConversationsPreparer()
    @recorded_by_proxy
    def test_swap_deployments(self, authoring_endpoint, authoring_key):
        authoring_client = self.create_client(authoring_endpoint, authoring_key)
        project_name = "EmailApp"
        deployment_name_1 = "staging"
        deployment_name_2 = "production"

        project_client = authoring_client.get_project_client(project_name)

        details = SwapDeploymentsDetails(
            first_deployment_name=deployment_name_1,
            second_deployment_name=deployment_name_2,
        )

        # Act: begin swap and wait for completion
        poller = project_client.project.begin_swap_deployments(body=details)
        try:
            poller.result()  # completes with None; raises on failure
        except HttpResponseError as e:
            print(f"Operation failed: {e.message}")
            print(e.error)
            raise

        # Success -> poller completed
        print(f"Swap deployments completed. done={poller.done()} status={poller.status()}")



import functools
import pytest
from datetime import date, datetime

from devtools_testutils import EnvironmentVariableLoader, AzureRecordedTestCase
from devtools_testutils.aio import recorded_by_proxy_async
from azure.core.credentials import AzureKeyCredential
from azure.ai.language.conversations.authoring.aio import ConversationAuthoringClient
from azure.ai.language.conversations.authoring.models import (
    AssignedProjectDeploymentsMetadata,
    AssignedProjectDeploymentMetadata,
)

ConversationsPreparer = functools.partial(
    EnvironmentVariableLoader,
    "authoring",
    authoring_endpoint="https://Sanitized.cognitiveservices.azure.com/",
    authoring_key="fake_key",
)


class TestConversationsAsync(AzureRecordedTestCase):
    async def create_client(self, endpoint: str, key: str) -> ConversationAuthoringClient:
        return ConversationAuthoringClient(endpoint, AzureKeyCredential(key))


class TestConversationsDeployProjectAsync(TestConversationsAsync):
    @ConversationsPreparer()
    @recorded_by_proxy_async
    @pytest.mark.asyncio
    async def test_list_assigned_resource_deployments_async(self, authoring_endpoint, authoring_key):
        client = await self.create_client(authoring_endpoint, authoring_key)
        async with client:
            # Act
            paged = client.list_assigned_resource_deployments()

            # Assert
            async for meta in paged:
                assert isinstance(meta, AssignedProjectDeploymentsMetadata)
                assert isinstance(meta.project_name, str) and meta.project_name
                assert isinstance(meta.deployments_metadata, list)

                for d in meta.deployments_metadata:
                    assert isinstance(d, AssignedProjectDeploymentMetadata)
                    assert isinstance(d.deployment_name, str) and d.deployment_name
                    assert isinstance(d.last_deployed_on, datetime)
                    assert isinstance(d.deployment_expires_on, date)



# pylint: disable=line-too-long,useless-suppression
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------
"""Customize generated code here.
Follow our quickstart for examples: https://aka.ms/azsdk/python/dpcodegen/python/customize
"""
from collections.abc import MutableMapping, Awaitable  # pylint:disable=import-error
from typing import Any, Callable, Tuple, TypeVar, cast, Mapping, Optional, List

import base64
import functools
import time

from azure.core.exceptions import ResourceNotFoundError
from azure.core.pipeline import PipelineResponse
from azure.core.polling import AsyncPollingMethod, PollingMethod
from azure.core.polling.base_polling import (
    BadResponse,
    LongRunningOperation,
    OperationFailed,
)
from azure.core.rest import HttpRequest

from ._enums import ExportedProjectFormat
from ._models import (
    AssignedProjectResource,
    CopyProjectDetails,
    CopyProjectState,
    ConversationExportedEntity,
    ConversationExportedIntent,
    ConversationExportedProjectAsset,
    ConversationExportedUtterance,
    CreateDeploymentDetails as _GeneratedCreateDeploymentDetails,
    DeploymentDeleteFromResourcesState,
    DeploymentState,
    EvaluationJobResult,
    EvaluationState,
    ExportProjectState,
    ExportedModelDetails,
    ExportedModelState,
    ExportedProject,
    ExportedUtteranceEntityLabel,
    ImportProjectState,
    LoadSnapshotState,
    ProjectDeletionState,
    ResourceMetadata,
    SwapDeploymentsDetails,
    SwapDeploymentsState,
    TrainingJobDetails,
)

JSON = MutableMapping[str, Any]
T = TypeVar("T")
PollingReturnType_co = TypeVar("PollingReturnType_co", covariant=True)

class _JobsStrategy(LongRunningOperation):
    """Interprets job-status responses and tells the poller which URL to use."""

    def __init__(self, async_url: str):
        self._async_url = async_url

    # We can poll if we have an operation-location URL
    def can_poll(self, pipeline_response: PipelineResponse) -> bool:  # noqa: D401
        return bool(self._async_url)

    # Always poll the jobs URL from operation-location
    def get_polling_url(self) -> str:  # noqa: D401
        return self._async_url

    # Initial status after the first call (202→InProgress, 200 could be immediate success)
    def set_initial_status(self, pipeline_response: PipelineResponse) -> str:  # noqa: D401
        sc = pipeline_response.http_response.status_code
        if sc in (200, 201):
            return "InProgress"  # job hasn’t finished yet; service will report status on poll
        if sc in (202, 204):
            return "InProgress"
        raise OperationFailed("Operation failed or canceled")

    # Map service statuses to core strings
    def get_status(self, pipeline_response: JSON) -> str:  # type: ignore
        raw = str(pipeline_response.get("status", "")).lower()
        if raw in ("succeeded",):
            return "Succeeded"
        if raw in ("failed",):
            return "Failed"
        if raw in ("cancelled", "canceled"):
            return "Canceled"
        # notstarted, running, cancelling, partiallycompleted → still in progress
        return "InProgress"

    # Force the final GET to also come from operation-location (or return None to use last body)
    def get_final_get_url(self, pipeline_response: PipelineResponse) -> Optional[str]:  # noqa: D401
        return self._async_url


class _JobsPollingMethod(PollingMethod):
    def __init__(self, polling_interval: float = 30.0, *, path_format_arguments: Optional[dict] = None, **kwargs: Any):
        self._polling_interval = polling_interval
        self._kwargs = kwargs
        self._path_format_arguments = path_format_arguments or {}

        # predeclare attributes to satisfy pylint W0201
        self._client: Any = None
        self._initial_response: Optional[PipelineResponse] = None
        self._deserialization_callback: Optional[Callable] = None
        self._resource: Optional[PipelineResponse] = None
        self._status: str = "NotStarted"
        self._operation: Any = None  # or a concrete type if available
        self._command: Optional[Callable[[], PipelineResponse]] = None

    # ---- LRO lifecycle ----
    def initialize(self, client: Any, initial_response: PipelineResponse, deserialization_callback: Callable) -> None:
        self._client = client
        self._initial_response = initial_response
        self._deserialization_callback = deserialization_callback
        self._resource = None
        self._status = "InProgress"

        # Extract operation-location (case-insensitive)
        headers = initial_response.http_response.headers
        op_loc = headers.get("Operation-Location") or headers.get("operation-location")
        if not op_loc:
            raise BadResponse("Missing Operation-Location header for job polling")

        # Resolve {Endpoint} etc.
        if self._path_format_arguments:
            op_loc = self._client.format_url(op_loc, **self._path_format_arguments)

        # Strategy: always use jobs URL
        self._operation = _JobsStrategy(op_loc)
        if not self._operation.can_poll(initial_response):
            raise BadResponse("Cannot poll: no jobs URL")

        # Command to GET status from jobs URL
        self._command = functools.partial(self._do_get, self._operation.get_polling_url())

        # Initial status
        self._status = self._operation.set_initial_status(initial_response)

    def run(self) -> None:
        while not self.finished():
            self.update_status()
            if not self.finished():
                time.sleep(self._polling_interval)

        # Final GET (via jobs URL) if strategy asks for it
        final_url = self._operation.get_final_get_url(self._initial_response)
        if final_url:
            self._resource = self._do_get(final_url)

    def finished(self) -> bool:
        return self._status in ("Succeeded", "Failed", "Canceled")

    def status(self) -> str:
        return self._status

    def resource(self) -> Any:
        if self._deserialization_callback is None or self._initial_response is None:
            raise RuntimeError("Polling method not initialized; call initialize() first.")
        # Return typed object using provided callback (expects PipelineResponse)
        return self._deserialization_callback(self._resource or self._initial_response)

    def update_status(self) -> None:
        if self._command is None:
            raise RuntimeError("Polling method not initialized; call initialize() first.")

        try:
            self._resource = self._command()
        except ResourceNotFoundError:
            # Treat as transient if your service uses 404-before-ready semantics
            self._resource = None

        body: JSON = {}
        if self._resource is not None:
            try:
                body = cast(JSON, self._resource.http_response.json())
            except Exception as exc:
                raise BadResponse("Polling response is not JSON") from exc
        self._status = self._operation.get_status(body)

    # ---- Helpers ----
    def _do_get(self, url: str) -> PipelineResponse:
        # REST pipeline path (new core)
        if hasattr(self._client, "send_request"):
            req = HttpRequest("GET", url)
            return cast(
                PipelineResponse, self._client.send_request(req, _return_pipeline_response=True, **self._kwargs)
            )
        # Legacy pipeline fallback
        request = self._client.get(url)
        return cast(
            PipelineResponse,
            self._client._pipeline.run(request, stream=False, **self._kwargs),  # pylint: disable=protected-access
        )

    # ---- Continuation token support (doc pattern) ----
    def get_continuation_token(self) -> str:
        import json

        if not self._operation:
            raise RuntimeError("Polling method not initialized")

        token = {
            "version": 1,
            "data": {
                "polling_url": self._operation.get_polling_url(),
                "polling_interval": self._polling_interval,
                "path_format_arguments": self._path_format_arguments,
            },
        }

        raw = json.dumps(token, separators=(",", ":"), ensure_ascii=True).encode("utf-8")
        return base64.b64encode(raw).decode("ascii")

    @classmethod
    def from_continuation_token(cls, continuation_token: str, **kwargs: Any) -> Tuple[Any, Any, Callable[[Any], PollingReturnType_co]]:
        try:
            client = kwargs["client"]
        except KeyError as exc:
            raise ValueError("Need kwarg 'client' to be recreated from continuation_token") from exc

        try:
            deserialization_callback = kwargs["deserialization_callback"]
        except KeyError as exc:
            raise ValueError(
                "Need kwarg 'deserialization_callback' to be recreated from continuation_token"
            ) from exc

        import json

        token = json.loads(base64.b64decode(continuation_token).decode("utf-8"))

        # Validate token schema and version for compatibility
        if not isinstance(token, dict) or "version" not in token or "data" not in token:
            raise ValueError("Invalid continuation token format.")
        if token["version"] != 1:
            raise ValueError(
                "This continuation token is not compatible with this version. "
                "It may have been generated by a different version."
            )

        # Extract the state from the "data" field
        state = token["data"]
        # The file_id and other state can be extracted and used to resume polling
        return client, state, deserialization_callback

class _AsyncJobsPollingMethod(AsyncPollingMethod):
    def __init__(
        self,
        polling_interval: float = 30.0,
        *,
        path_format_arguments: Optional[dict] = None,
        **kwargs: Any,
    ):
        self._polling_interval = polling_interval
        self._kwargs = kwargs
        self._path_format_arguments = path_format_arguments or {}

        # Predeclare all attributes to satisfy pylint W0201
        self._client: Any = None
        self._initial_response: Optional[PipelineResponse] = None
        self._deserialization_callback: Optional[Callable] = None
        self._resource: Optional[PipelineResponse] = None
        self._status: str = "NotStarted"
        self._operation: Any = None
        self._command: Optional[Callable[[], Awaitable[PipelineResponse]]] = None

    # ---- LRO lifecycle ----
    def initialize(self, client: Any, initial_response: PipelineResponse, deserialization_callback: Callable) -> None:
        self._client = client
        self._initial_response = initial_response
        self._deserialization_callback = deserialization_callback
        self._resource = None  # no type annotation here
        self._status = "InProgress"  # no type annotation here

        # Operation-Location (case-insensitive)
        headers = initial_response.http_response.headers
        op_loc = headers.get("Operation-Location") or headers.get("operation-location")
        if not op_loc:
            raise BadResponse("Missing Operation-Location header for job polling")

        if self._path_format_arguments:
            op_loc = self._client.format_url(op_loc, **self._path_format_arguments)

        self._operation = _JobsStrategy(op_loc)
        if not self._operation.can_poll(initial_response):
            raise BadResponse("Cannot poll: no jobs URL")

        # async GET command bound to jobs URL
        self._command = functools.partial(self._do_get_async, self._operation.get_polling_url())

        # Seed initial status from first response
        self._status = self._operation.set_initial_status(initial_response)

    async def run(self) -> None:
        while not self.finished():
            await self.update_status()
            if not self.finished():
                await self._sleep(self._polling_interval)

        final_url = self._operation.get_final_get_url(self._initial_response)
        if final_url:
            self._resource = await self._do_get_async(final_url)

    async def _sleep(self, seconds: float) -> None:
        # Prefer the Azure Core transport's sleep (fast/no-op in playback)
        transport = getattr(self._client, "_transport", None) or getattr(
            getattr(self._client, "_pipeline", None), "_transport", None
        )
        if transport and hasattr(transport, "sleep"):
            await transport.sleep(seconds)
            return
        # Fallback for non-Azure transports (allowed per rule text)
        import asyncio  # pylint: disable=import-outside-toplevel, do-not-import-asyncio

        await asyncio.sleep(seconds)

    def finished(self) -> bool:
        return self._status in ("Succeeded", "Failed", "Canceled")

    def status(self) -> str:
        return self._status

    def resource(self) -> Any:
        if self._deserialization_callback is None or self._initial_response is None:
            raise RuntimeError("Polling method not initialized; call initialize() first.")
        return self._deserialization_callback(self._resource or self._initial_response)

    async def update_status(self) -> None:
        if self._command is None:
            raise RuntimeError("Polling method not initialized; call initialize() first.")

        try:
            self._resource = await self._command()
        except ResourceNotFoundError:
            self._resource = None

        body: dict = {}
        if self._resource is not None:
            try:
                body = cast(dict, self._resource.http_response.json())
            except Exception as exc:  # be explicit so pylint sees the chain
                raise BadResponse("Polling response is not JSON") from exc
        self._status = self._operation.get_status(body)

    # ---- Helpers ----
    async def _do_get_async(self, url: str) -> PipelineResponse:
        # New core: async pipeline client has async send_request
        if hasattr(self._client, "send_request"):
            req = HttpRequest("GET", url)
            return cast(
                PipelineResponse,
                await self._client.send_request(req, _return_pipeline_response=True, **self._kwargs),
            )
        # Legacy fallback (unlikely in modern azure-core)
        request = self._client.get(url)
        return cast(
            PipelineResponse,
            await self._client._pipeline.run(request, stream=False, **self._kwargs),  # pylint: disable=protected-access
        )

    # ---- Continuation token ----
    def get_continuation_token(self) -> str:
        import json

        if not self._operation:
            raise RuntimeError("Polling method not initialized")

        token = {
            "version": 1,
            "data": {
                "polling_url": self._operation.get_polling_url(),
                "polling_interval": self._polling_interval,
                "path_format_arguments": self._path_format_arguments,
            },
        }

        raw = json.dumps(token, separators=(",", ":"), ensure_ascii=True).encode("utf-8")
        return base64.b64encode(raw).decode("ascii")

    @classmethod
    def from_continuation_token(cls, continuation_token: str, **kwargs: Any) -> Tuple[Any, Any, Callable[[Any], PollingReturnType_co]]:
        try:
            client = kwargs["client"]
        except KeyError as exc:
            raise ValueError("Need kwarg 'client' to be recreated from continuation_token") from exc

        try:
            deserialization_callback = kwargs["deserialization_callback"]
        except KeyError as exc:
            raise ValueError(
                "Need kwarg 'deserialization_callback' to be recreated from continuation_token"
            ) from exc

        import json

        token = json.loads(base64.b64decode(continuation_token).decode("utf-8"))

        # Validate token schema and version for compatibility
        if not isinstance(token, dict) or "version" not in token or "data" not in token:
            raise ValueError("Invalid continuation token format.")
        if token["version"] != 1:
            raise ValueError(
                "This continuation token is not compatible with this version. "
                "It may have been generated by a different version."
            )

        # Extract the state from the "data" field
        state = token["data"]
        # The file_id and other state can be extracted and used to resume polling
        return client, state, deserialization_callback

class CreateDeploymentDetails(_GeneratedCreateDeploymentDetails):
    """Represents the options for creating or updating a project deployment.

    :ivar trained_model_label: Represents the trained model label.
    :vartype trained_model_label: str
    :ivar azure_resource_ids: Language or AIService resource IDs associated with this deployment.
     For service version 2025-11-15-preview, this is represented as a list of
     class:`AssignedProjectResource`. For service version 2025-11-01, it may be
     constructed from a list of resource ID strings.
    :vartype azure_resource_ids:
     list[~azure.ai.language.conversations.authoring.models.AssignedProjectResource] or list[str]
    """

    _azure_resource_ids_strings: Optional[List[str]]

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        # Case 1: mapping initializer → let generated code handle it
        if args and isinstance(args[0], Mapping):
            super().__init__(*args, **kwargs)
            self._azure_resource_ids_strings = None
            return

        azure_ids = kwargs.pop("azure_resource_ids", None)
        self._azure_resource_ids_strings = None

        if azure_ids is None:
            # nothing special, just call base
            super().__init__(*args, azure_resource_ids=None, **kwargs)
            return

        # If user passed AssignedProjectResource list → original behavior
        if all(isinstance(x, AssignedProjectResource) for x in azure_ids):
            super().__init__(*args, azure_resource_ids=azure_ids, **kwargs)
            return

        # If user passed plain strings → GA style, remember them and *don't*
        # try to turn them into AssignedProjectResource here.
        if all(isinstance(x, str) for x in azure_ids):
            self._azure_resource_ids_strings = list(azure_ids)
            super().__init__(*args, azure_resource_ids=None, **kwargs)
            return

        raise TypeError(
            "azure_resource_ids must be a list of str or a list of AssignedProjectResource."
        )

def patch_sdk():
    """Do not remove from this file.
    `patch_sdk` is a last resort escape hatch that allows you to do customizations
    you can't accomplish using the techniques described in
    https://aka.ms/azsdk/python/dpcodegen/python/customize
    """


__all__ = [
    "SwapDeploymentsDetails",
    "CopyProjectState",
    "ExportProjectState",
    "SwapDeploymentsState",
    "CreateDeploymentDetails",
    "DeploymentDeleteFromResourcesState",
    "DeploymentState",
    "ExportedModelDetails",
    "ExportedModelState",
    "LoadSnapshotState",
    "ProjectDeletionState",
    "ExportedProjectFormat",
    "ExportedProject",
    "ImportProjectState",
    "CopyProjectDetails",
    "TrainingJobDetails",
    "CopyProjectDetails",
    "EvaluationJobResult",
    "EvaluationState",
    "ConversationExportedProjectAsset",
    "ConversationExportedIntent",
    "ConversationExportedEntity",
    "ConversationExportedUtterance",
    "ExportedUtteranceEntityLabel",
    "ResourceMetadata",
]



# pylint: disable=line-too-long,useless-suppression
import functools
import pytest

from devtools_testutils import EnvironmentVariableLoader, AzureRecordedTestCase
from devtools_testutils.aio import recorded_by_proxy_async
from azure.core.credentials import AzureKeyCredential
from azure.core.exceptions import HttpResponseError
from azure.ai.language.conversations.authoring.aio import ConversationAuthoringClient
from azure.ai.language.conversations.authoring.models import ProjectResourceIds

ConversationsPreparer = functools.partial(
    EnvironmentVariableLoader,
    "authoring",
    authoring_endpoint="https://Sanitized.cognitiveservices.azure.com/",
    authoring_key="fake_key",
)


class TestConversationsAsync(AzureRecordedTestCase):
    async def create_client(self, endpoint: str, key: str) -> ConversationAuthoringClient:
        return ConversationAuthoringClient(endpoint, AzureKeyCredential(key))


class TestConversationsDeleteDeploymentFromResourcesAsync(TestConversationsAsync):
    @ConversationsPreparer()
    @recorded_by_proxy_async
    @pytest.mark.asyncio
    async def test_delete_deployment_from_resources_async(self, authoring_endpoint, authoring_key):
        client = await self.create_client(authoring_endpoint, authoring_key)
        async with client:
            project_name = "EmailApp"
            deployment_name = "deploysdk2"
            project_client = client.get_project_client(project_name)

            delete_body = ProjectResourceIds(
                azure_resource_ids=[
                    "/subscriptions/b72743ec-8bb3-453f-83ad-a53e8a50712e/resourceGroups/language-sdk-rg/providers/Microsoft.CognitiveServices/accounts/sdk-test-02"
                ]
            )

            poller = await project_client.deployment.begin_delete_deployment_from_resources(
                deployment_name=deployment_name,
                body=delete_body,
            )

            try:
                await poller.result()
            except HttpResponseError as e:
                print(f"Operation failed: {e.message}")
                print(e.error)
                raise

            # Assert completion
            assert poller.done()
            print(f"Delete completed. done={poller.done()} status={poller.status()}")



# coding=utf-8
# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------

"""
FILE: sample_assign_project_resources.py
DESCRIPTION:
    This sample demonstrates how to assign deployment resources to a Conversation Authoring project.

USAGE:
    python sample_assign_project_resources.py

REQUIRED ENV VARS (for AAD / DefaultAzureCredential):
    AZURE_CONVERSATIONS_AUTHORING_ENDPOINT
    AZURE_CLIENT_ID
    AZURE_TENANT_ID
    AZURE_CLIENT_SECRET

NOTE:
    If you want to use AzureKeyCredential instead, set:
      - AZURE_CONVERSATIONS_AUTHORING_ENDPOINT
      - AZURE_CONVERSATIONS_AUTHORING_KEY

OPTIONAL ENV VARS:
    PROJECT_NAME         # defaults to "<project-name>"
    RESOURCE_ID          # defaults to "<azure-resource-id>"
    RESOURCE_DOMAIN      # defaults to "<custom-domain>"
    RESOURCE_REGION      # defaults to "<region>"
"""

# [START conversation_authoring_assign_project_resources]
import os
from azure.identity import DefaultAzureCredential
from azure.core.exceptions import HttpResponseError
from azure.ai.language.conversations.authoring import ConversationAuthoringClient
from azure.ai.language.conversations.authoring.models import AssignProjectResourcesDetails, ResourceMetadata


def sample_assign_project_resources():
    # settings
    endpoint = os.environ["AZURE_CONVERSATIONS_AUTHORING_ENDPOINT"]

    project_name = os.environ.get("PROJECT_NAME", "<project-name>")
    resource_id = os.environ.get("RESOURCE_ID", "<azure-resource-id>")
    resource_domain = os.environ.get("RESOURCE_DOMAIN", "<custom-domain>")
    resource_region = os.environ.get("RESOURCE_REGION", "<region>")

    credential = DefaultAzureCredential()
    client = ConversationAuthoringClient(endpoint, credential=credential)
    project_client = client.get_project_client(project_name)

    body = AssignProjectResourcesDetails(
        metadata=[
            ResourceMetadata(
                azure_resource_id=resource_id,
                custom_domain=resource_domain,
                region=resource_region,
            )
        ]
    )

    # start assign (long-running operation)
    poller = project_client.project.begin_assign_project_resources(body=body)

    try:
        poller.result()
        print("Assign completed.")
        print(f"done: {poller.done()}")
        print(f"status: {poller.status()}")
    except HttpResponseError as e:
        print(f"Operation failed: {e.message}")
        print(e.error)


# [END conversation_authoring_assign_project_resources]


def main():
    sample_assign_project_resources()


if __name__ == "__main__":
    main()



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------
"""Customize generated code here.
Follow our quickstart for examples: https://aka.ms/azsdk/python/dpcodegen/python/customize
"""
from typing import Any, TYPE_CHECKING, Union, Optional
from azure.core.credentials import AzureKeyCredential
from ._client import ConversationAuthoringClient as AuthoringClientGenerated
from ._client import ConversationAuthoringProjectClient as AuthoringProjectClientGenerated
from .operations import DeploymentOperations, ExportedModelOperations, ProjectOperations, TrainedModelOperations

if TYPE_CHECKING:
    from azure.core.credentials_async import AsyncTokenCredential


class ConversationAuthoringProjectClient(AuthoringProjectClientGenerated):

    #: Deployment operations group
    deployment: DeploymentOperations
    #: Exported model operations group
    exported_model: ExportedModelOperations
    #: Project operations group
    project: ProjectOperations
    #: Trained model operations group
    trained_model: TrainedModelOperations

    def __init__(
        self,
        endpoint: str,
        credential: Union[AzureKeyCredential, "AsyncTokenCredential"],
        *,
        api_version: Optional[str] = None,
        project_name: str,
        **kwargs: Any,
    ) -> None:
        """Initialize a ConversationAuthoringProjectClient.

        :param str endpoint: Supported Cognitive Services endpoint, e.g.
            "https://<resource>.cognitiveservices.azure.com".
        :param credential: Credential used to authenticate requests to the service.
        :type credential: ~azure.core.credentials.AzureKeyCredential or
            ~azure.core.credentials_async.AsyncTokenCredential
        :keyword str api_version: The API version to use for this operation.
            Default value is ``2025-05-15-preview``.
            Note that overriding this default value may result in unsupported behavior.
        :keyword str project_name: The name of the project to scope operations. Required.
        """
        if api_version is not None:
            kwargs["api_version"] = api_version
        super().__init__(endpoint=endpoint, credential=credential, project_name=project_name, **kwargs)


class ConversationAuthoringClient(AuthoringClientGenerated):

    def __init__(
        self,
        endpoint: str,
        credential: Union[AzureKeyCredential, "AsyncTokenCredential"],
        *,
        api_version: Optional[str] = None,
        **kwargs: Any,
    ) -> None:
        """
        Create a ConversationAuthoringClient.

        :param str endpoint: Supported Cognitive Services endpoint, e.g.
            ``https://<resource-name>.api.cognitiveservices.azure.com``.
        :param credential: Key or token credential.
        :type credential: ~azure.core.credentials.AzureKeyCredential or
            ~azure.core.credentials_async.AsyncTokenCredential
        :keyword str api_version: The API version to use for this operation.
            Default value is ``2025-05-15-preview``.
            Note that overriding this default value may result in unsupported behavior.
        """
        if api_version is not None:
            kwargs["api_version"] = api_version
        super().__init__(endpoint=endpoint, credential=credential, **kwargs)

    def get_project_client(self, project_name: str) -> ConversationAuthoringProjectClient:
        return ConversationAuthoringProjectClient(
            endpoint=self._config.endpoint,
            credential=self._config.credential,
            project_name=project_name,
            api_version=self._config.api_version,
        )


def patch_sdk():
    """Do not remove from this file.
    `patch_sdk` is a last resort escape hatch that allows you to do customizations
    you can't accomplish using the techniques described in
    https://aka.ms/azsdk/python/dpcodegen/python/customize
    """


__all__ = ["ConversationAuthoringProjectClient", "ConversationAuthoringClient"]



# pylint: disable=line-too-long,useless-suppression
from datetime import date, datetime
import functools
from importlib import resources

from devtools_testutils import AzureRecordedTestCase, EnvironmentVariableLoader, recorded_by_proxy
from azure.core.credentials import AzureKeyCredential
from azure.ai.language.conversations.authoring import ConversationAuthoringClient
from azure.ai.language.conversations.authoring.models import (
    AssignedProjectDeploymentsMetadata,
    AssignedProjectDeploymentMetadata,
)

ConversationsPreparer = functools.partial(
    EnvironmentVariableLoader,
    "authoring",
    authoring_endpoint="https://Sanitized.cognitiveservices.azure.com/",
    authoring_key="fake_key",
)


class TestConversations(AzureRecordedTestCase):
    def create_client(self, endpoint, key):
        return ConversationAuthoringClient(endpoint, AzureKeyCredential(key))


class TestConversationsDeployProjectSync(TestConversations):
    @ConversationsPreparer()
    @recorded_by_proxy
    def test_list_assigned_resource_deployments(self, authoring_endpoint, authoring_key):
        # Arrange
        client = self.create_client(authoring_endpoint, authoring_key)
        paged = client.list_assigned_resource_deployments()

        # Assert
        for meta in paged:
            assert isinstance(meta, AssignedProjectDeploymentsMetadata)
            assert isinstance(meta.project_name, str) and meta.project_name
            assert isinstance(meta.deployments_metadata, list)

            for d in meta.deployments_metadata:
                assert isinstance(d, AssignedProjectDeploymentMetadata)
                assert isinstance(d.deployment_name, str) and d.deployment_name
                assert isinstance(d.last_deployed_on, datetime)
                assert isinstance(d.deployment_expires_on, date)



# pylint: disable=line-too-long,useless-suppression
import functools
import pytest

from devtools_testutils import AzureRecordedTestCase, EnvironmentVariableLoader, recorded_by_proxy
from azure.core.credentials import AzureKeyCredential
from azure.ai.language.conversations.authoring import ConversationAuthoringClient
from azure.ai.language.conversations.authoring.models import StringIndexType

ConversationsPreparer = functools.partial(
    EnvironmentVariableLoader,
    "authoring",
    authoring_endpoint="https://Sanitized.cognitiveservices.azure.com/",
    authoring_key="fake_key",
)


class TestConversations(AzureRecordedTestCase):
    def create_client(self, endpoint, key):
        return ConversationAuthoringClient(endpoint, AzureKeyCredential(key))


class TestConversationsGetModelEvaluationResultsSync(TestConversations):
    @ConversationsPreparer()
    @recorded_by_proxy
    def test_get_model_evaluation_results(self, authoring_endpoint, authoring_key):
        authoring_client = self.create_client(authoring_endpoint, authoring_key)

        project_name = "Aurora-CLU-Prod"
        trained_model_label = "m1"

        # Get trained-model scoped client and fetch the paged evaluation results
        project_client = authoring_client.get_project_client(project_name)
        results = project_client.trained_model.get_model_evaluation_results(
            trained_model_label=trained_model_label, string_index_type="Utf16CodeUnit"
        )

        # Iterate and print like the C# sample
        seen = 0
        for r in results:
            seen += 1
            print(f"Text: {getattr(r, 'text', None)}")
            print(f"Language: {getattr(r, 'language', None)}")

            intents_result = getattr(r, "intents_result", None)
            if intents_result:
                print(f"Expected Intent: {getattr(intents_result, 'expected_intent', None)}")
                print(f"Predicted Intent: {getattr(intents_result, 'predicted_intent', None)}")

            entities_result = getattr(r, "entities_result", None)
            if entities_result:
                expected_entities = getattr(entities_result, "expected_entities", []) or []
                print("Expected Entities:")
                for ent in expected_entities:
                    print(
                        f" - Category: {getattr(ent, 'category', None)}, Offset: {getattr(ent, 'offset', None)}, Length: {getattr(ent, 'length', None)}"
                    )

                predicted_entities = getattr(entities_result, "predicted_entities", []) or []
                print("Predicted Entities:")
                for ent in predicted_entities:
                    print(
                        f" - Category: {getattr(ent, 'category', None)}, Offset: {getattr(ent, 'offset', None)}, Length: {getattr(ent, 'length', None)}"
                    )

                print()

        # Basic assertion that call succeeded (pageable iterated, possibly empty)
        assert results is not None



# pylint: disable=line-too-long,useless-suppression
import functools
from importlib import resources

from devtools_testutils import AzureRecordedTestCase, EnvironmentVariableLoader, recorded_by_proxy
from azure.core.credentials import AzureKeyCredential
from azure.ai.language.conversations.authoring import ConversationAuthoringClient
from azure.ai.language.conversations.authoring.models import AssignedProjectResource

ConversationsPreparer = functools.partial(
    EnvironmentVariableLoader,
    "authoring",
    authoring_endpoint="https://Sanitized.cognitiveservices.azure.com/",
    authoring_key="fake_key",
)


class TestConversations(AzureRecordedTestCase):
    def create_client(self, endpoint, key):
        return ConversationAuthoringClient(endpoint, AzureKeyCredential(key))


class TestConversationsListProjectResourcesSync(TestConversations):
    @ConversationsPreparer()
    @recorded_by_proxy
    def test_list_project_resources(self, authoring_endpoint, authoring_key):
        # Arrange
        client = self.create_client(authoring_endpoint, authoring_key)
        project_name = "EmailApp"
        project_client = client.get_project_client(project_name)

        # Act
        paged = project_client.project.list_project_resources()

        # Assert each resource item
        for res in paged:
            assert isinstance(res, AssignedProjectResource)
            assert isinstance(res.resource_id, str) and res.resource_id
            assert isinstance(res.region, str) and res.region



# coding=utf-8
# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------

"""
FILE: sample_swap_deployments.py
DESCRIPTION:
    This sample demonstrates how to swap two deployments within a Conversation Authoring project.
USAGE:
    python sample_swap_deployments.py

REQUIRED ENV VARS (for AAD / DefaultAzureCredential):
    AZURE_CONVERSATIONS_AUTHORING_ENDPOINT
    AZURE_CLIENT_ID
    AZURE_TENANT_ID
    AZURE_CLIENT_SECRET

NOTE:
    If you want to use AzureKeyCredential instead, set:
      - AZURE_CONVERSATIONS_AUTHORING_ENDPOINT
      - AZURE_CONVERSATIONS_AUTHORING_KEY

OPTIONAL ENV VARS:
    PROJECT_NAME          # defaults to "<project-name>"
    FIRST_DEPLOYMENT      # defaults to "<first-deployment>"
    SECOND_DEPLOYMENT     # defaults to "<second-deployment>"
"""

# [START conversation_authoring_swap_deployments]
import os
from azure.identity import DefaultAzureCredential
from azure.core.exceptions import HttpResponseError
from azure.ai.language.conversations.authoring import ConversationAuthoringClient
from azure.ai.language.conversations.authoring.models import SwapDeploymentsDetails


def sample_swap_deployments():
    # settings
    endpoint = os.environ["AZURE_CONVERSATIONS_AUTHORING_ENDPOINT"]
    project_name = os.environ.get("PROJECT_NAME", "<project-name>")
    deployment_name_1 = os.environ.get("FIRST_DEPLOYMENT", "<first-deployment>")
    deployment_name_2 = os.environ.get("SECOND_DEPLOYMENT", "<second-deployment>")

    # create a client with AAD
    credential = DefaultAzureCredential()
    client = ConversationAuthoringClient(endpoint, credential=credential)
    project_client = client.get_project_client(project_name)

    # build swap details
    details = SwapDeploymentsDetails(
        first_deployment_name=deployment_name_1,
        second_deployment_name=deployment_name_2,
    )

    # start swap (long-running operation)
    poller = project_client.project.begin_swap_deployments(body=details)

    try:
        poller.result()
        print("Swap completed.")
        print(f"done: {poller.done()}")
        print(f"status: {poller.status()}")
    except HttpResponseError as e:
        print(f"Operation failed: {e.message}")
        print(e.error)


# [END conversation_authoring_swap_deployments]


def main():
    sample_swap_deployments()


if __name__ == "__main__":
    main()



# pylint: disable=line-too-long,useless-suppression
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------
"""Customize generated code here.
Follow our quickstart for examples: https://aka.ms/azsdk/python/dpcodegen/python/customize
"""
from collections.abc import MutableMapping  # pylint:disable=import-error
from typing import Any, Callable, Dict, Optional, TypeVar, Union, cast

from azure.core.pipeline import PipelineResponse
from azure.core.polling import AsyncLROPoller, AsyncNoPolling, AsyncPollingMethod
from azure.core.rest import HttpRequest, HttpResponse
from azure.core.tracing.decorator_async import distributed_trace_async

from ..._utils.model_base import _deserialize
from ...models._patch import _AsyncJobsPollingMethod
from ...models import (
    TrainingJobResult,
)
from ._operations import ProjectOperations as ProjectOperationsGenerated

JSON = MutableMapping[str, Any]
T = TypeVar("T")
_Unset: Any = object()
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, HttpResponse], T, Dict[str, Any]], Any]]


class ProjectOperations(ProjectOperationsGenerated):
    """Patched ProjectOperationsOperations that auto-injects project_name."""

    @distributed_trace_async
    async def begin_cancel_training_job(  # type: ignore[override]
        self, job_id: str, **kwargs: Any
    ) -> AsyncLROPoller[TrainingJobResult]:
        """
        Cancel a training job.
        :param job_id: The identifier of the training job to cancel. Required.
        :type job_id: str
        :return: An instance of AsyncLROPoller that returns TrainingJobResult.
        :rtype: ~azure.core.polling.AsyncLROPoller[~azure.ai.language.conversations.authoring.models.TrainingJobResult]
        """
        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[TrainingJobResult] = kwargs.pop("cls", None)
        polling: Union[bool, AsyncPollingMethod] = kwargs.pop("polling", True)
        lro_delay = kwargs.pop("polling_interval", self._config.polling_interval)
        cont_token: Optional[str] = kwargs.pop("continuation_token", None)

        if cont_token is None:
            # Fire the initial cancel request; keep PipelineResponse for the poller
            initial = await self._cancel_training_job_initial(  # returns PipelineResponse                project_name=self._project_name,
                job_id=job_id,
                cls=lambda x, y, z: x,  # passthrough PipelineResponse
                headers=_headers,
                params=_params,
                **kwargs,
            )
            await initial.http_response.read()  # type: ignore[attr-defined]
        kwargs.pop("error_map", None)

        # Deserialization callback: map the nested "result" object to TrainingJobResult
        def get_long_running_output(pipeline_response):
            body = pipeline_response.http_response.json() or {}
            result_dict = body.get("result", {}) or {}
            obj = _deserialize(TrainingJobResult, result_dict)
            if cls:
                return cls(pipeline_response, obj, {})  # type: ignore[misc]
            return obj

        path_format_arguments = {
            "Endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }

        if polling is True:
            polling_method: AsyncPollingMethod = cast(
                AsyncPollingMethod,
                _AsyncJobsPollingMethod(
                    polling_interval=lro_delay,
                    path_format_arguments=path_format_arguments,
                ),
            )
        elif polling is False:
            polling_method = cast(AsyncPollingMethod, AsyncNoPolling())
        else:
            polling_method = polling

        if cont_token:
            return AsyncLROPoller[TrainingJobResult].from_continuation_token(
                polling_method=polling_method,
                continuation_token=cont_token,
                client=self._client,
                deserialization_callback=get_long_running_output,
            )

        return AsyncLROPoller[TrainingJobResult](
            self._client,
            initial,  # type: ignore
            get_long_running_output,
            polling_method,  # type: ignore[arg-type]
        )



# pylint: disable=line-too-long,useless-suppression
import functools

from devtools_testutils import AzureRecordedTestCase, EnvironmentVariableLoader, recorded_by_proxy
from azure.core.credentials import AzureKeyCredential
from azure.ai.language.conversations.authoring import ConversationAuthoringClient
from azure.ai.language.conversations.authoring.models import ProjectDetails

ConversationsPreparer = functools.partial(
    EnvironmentVariableLoader,
    "authoring",
    authoring_endpoint="https://Sanitized.cognitiveservices.azure.com/",
    authoring_key="fake_key",
)


class TestConversations(AzureRecordedTestCase):
    def create_client(self, endpoint, key):
        return ConversationAuthoringClient(endpoint, AzureKeyCredential(key))


class TestConversationsGetProjectSync(TestConversations):
    @ConversationsPreparer()
    @recorded_by_proxy
    def test_get_project(self, authoring_endpoint, authoring_key):
        authoring_client = self.create_client(authoring_endpoint, authoring_key)
        project_name = "EmailApp"

        # Act
        details = authoring_client.get_project(project_name=project_name)

        # Assert
        assert isinstance(details, ProjectDetails)
        assert details.project_name == project_name

        # Print like the C# sample
        print(f"Created DateTime: {details.created_on}")
        print(f"Last Modified DateTime: {details.last_modified_on}")
        print(f"Last Trained DateTime: {details.last_trained_on}")
        print(f"Last Deployed DateTime: {details.last_deployed_on}")
        print(f"Project Kind: {details.project_kind}")
        print(f"Project Name: {details.project_name}")
        print(f"Multilingual: {details.multilingual}")
        print(f"Description: {details.description}")
        print(f"Language: {details.language}")



# coding=utf-8
# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------

"""
FILE: sample_delete_trained_model.py
DESCRIPTION:
    This sample demonstrates how to delete a trained model from a Conversation Authoring project.
USAGE:
    python sample_delete_trained_model.py

REQUIRED ENV VARS (for AAD / DefaultAzureCredential):
    AZURE_CONVERSATIONS_AUTHORING_ENDPOINT
    AZURE_CLIENT_ID
    AZURE_TENANT_ID
    AZURE_CLIENT_SECRET

NOTE:
    If you want to use AzureKeyCredential instead, set:
      - AZURE_CONVERSATIONS_AUTHORING_ENDPOINT
      - AZURE_CONVERSATIONS_AUTHORING_KEY

OPTIONAL ENV VARS:
    PROJECT_NAME       # defaults to "<project-name>"
    TRAINED_MODEL      # defaults to "<trained-model-label>"
"""

# [START conversation_authoring_delete_trained_model]
import os
from azure.identity import DefaultAzureCredential
from azure.ai.language.conversations.authoring import ConversationAuthoringClient


def sample_delete_trained_model():
    # settings
    endpoint = os.environ["AZURE_CONVERSATIONS_AUTHORING_ENDPOINT"]
    project_name = os.environ.get("PROJECT_NAME", "<project-name>")
    trained_model_label = os.environ.get("TRAINED_MODEL", "<trained-model-label>")

    # create a client with AAD
    credential = DefaultAzureCredential()
    client = ConversationAuthoringClient(endpoint, credential=credential)
    project_client = client.get_project_client(project_name)

    captured = {}

    def capture_response(pipeline_response):
        # capture the raw HTTP response status code
        captured["status_code"] = pipeline_response.http_response.status_code

    # delete trained model
    project_client.trained_model.delete_trained_model(
        trained_model_label,
        raw_response_hook=capture_response,
    )

    # print response
    status = captured.get("status_code")
    print(f"Delete Trained Model Response Status: {status}")


# [END conversation_authoring_delete_trained_model]


def main():
    sample_delete_trained_model()


if __name__ == "__main__":
    main()



# coding=utf-8
# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------

"""
FILE: sample_create_project.py
DESCRIPTION:
    This sample demonstrates how to create a Conversation Authoring project.
USAGE:
    python sample_create_project.py

REQUIRED ENV VARS (for AAD / DefaultAzureCredential):
    AZURE_CONVERSATIONS_AUTHORING_ENDPOINT
    AZURE_CLIENT_ID
    AZURE_TENANT_ID
    AZURE_CLIENT_SECRET

NOTE:
    If you want to use AzureKeyCredential instead, set:
      - AZURE_CONVERSATIONS_AUTHORING_ENDPOINT
      - AZURE_CONVERSATIONS_AUTHORING_KEY

OPTIONAL ENV VARS:
    PROJECT_NAME   # defaults to "<project-name>"
"""

# [START conversation_authoring_create_project]
import os
from azure.identity import DefaultAzureCredential
from azure.ai.language.conversations.authoring import ConversationAuthoringClient
from azure.ai.language.conversations.authoring.models import (
    CreateProjectOptions,
    ProjectKind,
)


def sample_create_project():
    # settings
    endpoint = os.environ["AZURE_CONVERSATIONS_AUTHORING_ENDPOINT"]
    project_name = os.environ.get("PROJECT_NAME", "<project-name>")

    # create a client with AAD
    credential = DefaultAzureCredential()
    client = ConversationAuthoringClient(endpoint, credential=credential)

    # build project definition
    body = CreateProjectOptions(
        project_kind=ProjectKind.CONVERSATION,
        project_name=project_name,
        language="<language-tag>",  # e.g. "en-us"
        multilingual=True,
        description="Sample project created via Python SDK",
    )

    # create project
    result = client.create_project(project_name=project_name, body=body)

    # print project details (direct attribute access; no getattr)
    print("=== Create Project Result ===")
    print(f"Project Name: {result.project_name}")
    print(f"Language: {result.language}")
    print(f"Kind: {result.project_kind}")
    print(f"Multilingual: {result.multilingual}")
    print(f"Description: {result.description}")


# [END conversation_authoring_create_project]


def main():
    sample_create_project()


if __name__ == "__main__":
    main()



# coding=utf-8
# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------

"""
FILE: sample_list_project_resources.py
DESCRIPTION:
    This sample demonstrates how to list all assigned resources for a Conversation Authoring project.

USAGE:
    python sample_list_project_resources.py

REQUIRED ENV VARS (for AAD / DefaultAzureCredential):
    AZURE_CONVERSATIONS_AUTHORING_ENDPOINT
    AZURE_CLIENT_ID
    AZURE_TENANT_ID
    AZURE_CLIENT_SECRET

NOTE:
    If you want to use AzureKeyCredential instead, set:
      - AZURE_CONVERSATIONS_AUTHORING_ENDPOINT
      - AZURE_CONVERSATIONS_AUTHORING_KEY

OPTIONAL ENV VARS:
    PROJECT_NAME         # defaults to "<project-name>"
"""

# [START conversation_authoring_list_project_resources]
import os
from azure.identity import DefaultAzureCredential
from azure.core.exceptions import HttpResponseError
from azure.ai.language.conversations.authoring import ConversationAuthoringClient
from azure.ai.language.conversations.authoring.models import AssignedProjectResource


def sample_list_project_resources():
    # settings
    endpoint = os.environ["AZURE_CONVERSATIONS_AUTHORING_ENDPOINT"]
    project_name = os.environ.get("PROJECT_NAME", "<project-name>")

    credential = DefaultAzureCredential()
    client = ConversationAuthoringClient(endpoint, credential=credential)
    project_client = client.get_project_client(project_name)

    # list assigned project resources
    try:
        paged = project_client.project.list_project_resources()
        print("Assigned project resources retrieved successfully.")

        for res in paged:
            print(f"\nResource ID: {res.resource_id}")
            print(f"Region: {res.region}")
    except HttpResponseError as e:
        print(f"Operation failed: {e.message}")
        print(e.error)


# [END conversation_authoring_list_project_resources]


def main():
    sample_list_project_resources()


if __name__ == "__main__":
    main()



# coding=utf-8

# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------

"""
FILE: sample_authentication.py

DESCRIPTION:
    This sample demonstrates how to authenticate to the Conversational Language Understanding
    **Authoring** service.
    We authenticate using an AzureKeyCredential from azure.core.credentials or a token credential from the
    azure-identity client library.

    See more details about authentication here:
    https://learn.microsoft.com/azure/cognitive-services/authentication

    Note: the endpoint must be formatted to use the custom domain name for your resource:
    https://<NAME-OF-YOUR-RESOURCE>.cognitiveservices.azure.com/

USAGE:
    python sample_authentication.py

    Set the environment variables with your own values before running the sample:

    For API Key authentication:
        1) AZURE_CONVERSATIONS_ENDPOINT - the endpoint to your Conversational Language Understanding resource
        2) AZURE_CONVERSATIONS_KEY      - your Conversational Language Understanding API key

    For Azure Active Directory (DefaultAzureCredential) authentication:
        1) AZURE_CONVERSATIONS_ENDPOINT - the endpoint to your Conversational Language Understanding resource
        2) AZURE_CLIENT_ID              - your Azure Active Directory application client ID
        3) AZURE_TENANT_ID              - your Azure Active Directory tenant ID
        4) AZURE_CLIENT_SECRET          - your Azure Active Directory client secret
"""

import os


def sample_authentication_api_key():
    # [START create_dt_client_with_key]
    from azure.core.credentials import AzureKeyCredential
    from azure.ai.language.conversations.authoring import ConversationAuthoringClient

    endpoint = os.environ["AZURE_CONVERSATIONS_ENDPOINT"]
    key = os.environ["AZURE_CONVERSATIONS_KEY"]

    clu_client = ConversationAuthoringClient(endpoint, AzureKeyCredential(key))
    # [END create_clu_client_with_key]


def sample_authentication_with_azure_active_directory():
    """DefaultAzureCredential will use the values from these environment
    variables: AZURE_CLIENT_ID, AZURE_TENANT_ID, AZURE_CLIENT_SECRET
    """
    print("\n.. authentication_with_azure_active_directory")
    from azure.ai.language.conversations.authoring import ConversationAuthoringClient
    from azure.identity import DefaultAzureCredential

    endpoint = os.environ["AZURE_CONVERSATIONS_ENDPOINT"]
    credential = DefaultAzureCredential()

    clu_client = ConversationAuthoringClient(endpoint, credential=credential)


def main():
    sample_authentication_api_key()


if __name__ == "__main__":
    main()



# coding=utf-8
# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------

"""
FILE: sample_list_assigned_resource_deployments.py
DESCRIPTION:
    This sample demonstrates how to list all assigned resource deployments for Conversation Authoring projects.

USAGE:
    python sample_list_assigned_resource_deployments.py

REQUIRED ENV VARS (for AAD / DefaultAzureCredential):
    AZURE_CONVERSATIONS_AUTHORING_ENDPOINT
    AZURE_CLIENT_ID
    AZURE_TENANT_ID
    AZURE_CLIENT_SECRET

NOTE:
    If you want to use AzureKeyCredential instead, set:
      - AZURE_CONVERSATIONS_AUTHORING_ENDPOINT
      - AZURE_CONVERSATIONS_AUTHORING_KEY
"""

# [START conversation_authoring_list_assigned_resource_deployments]
import os
from datetime import date, datetime
from azure.identity import DefaultAzureCredential
from azure.core.exceptions import HttpResponseError
from azure.ai.language.conversations.authoring import ConversationAuthoringClient


def sample_list_assigned_resource_deployments():
    # settings
    endpoint = os.environ["AZURE_CONVERSATIONS_AUTHORING_ENDPOINT"]

    credential = DefaultAzureCredential()
    client = ConversationAuthoringClient(endpoint, credential=credential)

    # list assigned resource deployments
    try:
        paged = client.list_assigned_resource_deployments()
        print("Assigned resource deployments retrieved successfully.")

        for meta in paged:
            print(f"\nProject Name: {meta.project_name}")
            for d in meta.deployments_metadata:
                print(f"  Deployment Name: {d.deployment_name}")
                print(f"  Last Deployed On: {d.last_deployed_on}")
                print(f"  Expires On: {d.deployment_expires_on}")
    except HttpResponseError as e:
        print(f"Operation failed: {e.message}")
        print(e.error)


# [END conversation_authoring_list_assigned_resource_deployments]


def main():
    sample_list_assigned_resource_deployments()


if __name__ == "__main__":
    main()
